# -*- encoding: utf-8 -*-
"""
This module contents initialization of the Celery instance - a distributed task queue.

The `Celery` file structure consists of a few files.
termidesk/
        src/
            server/
                celery.py - this module
                celeryconfig.py - configuration file for the `Celery` instance

Configuration parameter reads from `settings.py`, and then from `celery_utils.py`.

"""
from __future__ import absolute_import

import os

from celery import Celery

# set the default Django settings module for the 'celery' program.
os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'server.settings')

# TODO: Delete if not needed
# sys.path.append('/'.join((os.path.dirname(settings.BASE_DIR), 'tests')))
# sys.path.append('/'.join((os.path.dirname(settings.BASE_DIR), 'tests/celery')))

# initialize the celery
app = Celery('server', )

# Using a string here means the worker doesn't have to serialize
# the configuration object to child processes.
# - namespace='CELERY' means all celery-related configuration keys
#   should have a `CELERY_` prefix.
app.config_from_object('server.celeryconfig')

# Load task modules from all registered Django app configs.
app.autodiscover_tasks()
