import logging

from rest_framework.generics import RetrieveAPIView
from rest_framework.schemas.openapi import AutoSchema
from uds.models import UserService
from termidesk.api.permissions import AccessByServicePoolPermission
from termidesk.api.fields import TypedSerializerMethodField
from .serializers.service import ServiceInfo
from .serializers.service import ServiceSettings

logger = logging.getLogger(__name__)

UUID_URL_KWARG = "uuid_actor"


class ServiceSettingsSchema(AutoSchema):
    def get_operation(self, path, method):
        operation = super().get_operation(path, method)
        uuid_param = [item for item in operation["parameters"] if item["name"] == UUID_URL_KWARG][0]
        uuid_param["description"] = "UUID рабочего места / агента"

        return operation

    def map_field(self, field):
        if isinstance(field, TypedSerializerMethodField):
            return AutoSchema.map_field(self, field.field_type())

        return AutoSchema.map_field(self, field)


class ServiceSettingsView(RetrieveAPIView):
    """
    Получение информации о настройках рабочего места
    """

    serializer_class = ServiceSettings
    permission_classes = [AccessByServicePoolPermission]
    schema = ServiceSettingsSchema(operation_id_base='AgentDraftServiceSettings')
    lookup_field = "uuid"
    lookup_url_kwarg = UUID_URL_KWARG

    def get_queryset(self):
        return UserService.objects.all()


class ServiceInfoSchema(AutoSchema):
    def get_operation(self, path, method):
        operation = super().get_operation(path, method)
        uuid_param = [item for item in operation["parameters"] if item["name"] == UUID_URL_KWARG][0]
        uuid_param["description"] = "UUID рабочего места / агента"

        return operation

    def map_field(self, field):
        if isinstance(field, TypedSerializerMethodField):
            return AutoSchema.map_field(self, field.field_type())

        return AutoSchema.map_field(self, field)


class ServiceInfoView(RetrieveAPIView):
    """
    Получение информации о рабочем месте (сервисе)
    """

    serializer_class = ServiceInfo
    permission_classes = [AccessByServicePoolPermission]
    schema = ServiceInfoSchema(operation_id_base='AgentDraftServiceInfo')
    lookup_field = "uuid"
    lookup_url_kwarg = UUID_URL_KWARG

    def get_queryset(self):
        return UserService.objects.all()
