import logging

from django.http import JsonResponse
from rest_framework.decorators import api_view, permission_classes
from rest_framework.generics import get_object_or_404
from rest_framework.response import Response
from rest_framework.generics import RetrieveAPIView
from rest_framework.schemas.openapi import AutoSchema
from termidesk.api.permissions import AccessByServicePoolPermission
from uds.models import UserService
from .serializers.service import ServiceInfo
from termidesk.api.fields import TypedSerializerMethodField

logger = logging.getLogger(__name__)

UUID_URL_KWARG = "uuid_actor"


@api_view(['GET'])
@permission_classes([AccessByServicePoolPermission])
def settings(request, uuid_actor):
    user_service = get_object_or_404(UserService, uuid=uuid_actor)

    return Response({'debug': user_service.deployed_service.debug_mode})


class ServiceInfoSchema(AutoSchema):
    def get_operation(self, path, method):
        operation = super().get_operation(path, method)
        uuid_param = [item for item in operation["parameters"] if item["name"] == UUID_URL_KWARG][0]
        uuid_param["description"] = "UUID рабочего места / агента"

        return operation

    def map_field(self, field):
        if isinstance(field, TypedSerializerMethodField):
            return AutoSchema.map_field(self, field.field_type())

        return AutoSchema.map_field(self, field)


class ServiceInfoView(RetrieveAPIView):
    """
    Получение информации о рабочем месте (сервисе)
    """

    serializer_class = ServiceInfo
    permission_classes = [AccessByServicePoolPermission]
    schema = ServiceInfoSchema(operation_id_base='AgentV1_1ServiceInfo')
    lookup_field = "uuid"
    lookup_url_kwarg = UUID_URL_KWARG

    def get_queryset(self):
        return UserService.objects.all()
