from rest_framework.schemas.openapi import AutoSchema
from .serializers.legacy_login import LegacyLoginErrorSerializer, LegacyLoginSuccessSerializer


import logging

logger = logging.getLogger(__name__)


class LegacyLoginSchema(AutoSchema):
    serializers_classes = [
        LegacyLoginSuccessSerializer,
        LegacyLoginErrorSerializer
    ]

    def get_components(self, path, method):
        # Получаем описание сериализатора указанного во view
        components = AutoSchema.get_components(self, path, method)
        
        # Добавляем описание сериализаторов для ответа
        for clazz in self.serializers_classes:
            serializer = clazz()
            component_name = self.get_component_name(serializer)
            content = self.map_serializer(serializer)
            components[component_name] = content
        return components

    def get_responses(self, path, method):
        responses = AutoSchema.get_responses(self, path, method)
        
        # Переопределяем код возврата для метода POST
        responses['200'] = responses['201']
        del(responses['201'])
        
        return responses

    def get_operation(self, path, method):
        # Получаем базовое описание операции
        res = AutoSchema.get_operation(self, path, method)

        # Получаем ссылки openapi на компоненты ответа
        refs = []
        for clazz in self.serializers_classes:
            serializer = clazz()
            refs.append(self._get_reference(serializer))

        # Заменяем описание схемы ответа
        schema = res['responses']['200']['content']['application/json']['schema']
        schema.clear()
        schema['oneOf'] = refs

        return res


class LegacyLogoutSchema(AutoSchema):

    def get_operation(self, path, method):
        # Получаем базовое описание операции
        res = AutoSchema.get_operation(self, path, method)
        
        # Выставляем механизм безопасности для корректной работы из интерфеса Swagger.
        # См. также модуль termidesk.api.swagger_schema
        res["security"] = [
            {
                "legacyTokenAuth": []
            }
        ]
        return res

    def get_responses(self, path, method):
        responses = AutoSchema.get_responses(self, path, method)
        
        if method == 'POST':
            # Переопределяем код возврата для метода POST
            responses['200'] = responses['201']
            del(responses['201'])
        
        return responses
