from rest_framework import serializers
from django.utils.translation import gettext_lazy as _
from django.contrib.sessions.backends.db import SessionStore
from uds.models.Authenticator import Authenticator
from uds.core.util.model import processUuid


class LegacyLoginRequestSerializer(serializers.Serializer):
    """ Сериалайзер для post запроса клиента на авторизацию. """
    username = serializers.CharField(
        label=_("Username"),
        help_text=("Логин пользователя"),
        allow_blank=True,
    )
    password = serializers.CharField(
        label=_("Password"),
        style={'input_type': 'password'},
        trim_whitespace=False,
        help_text=("Пароль пользователя"),
        allow_blank=True,
    )
    auth = serializers.CharField(
        label=_("Authenticator name"),
        help_text=("Название домена аутентификации"),
        required=False,
    )
    authId = serializers.UUIDField(
        label=_("Authenticator UUID"),
        help_text=("UUID домена аутентификации"),
        required=False,
    )
    authSmallName = serializers.CharField(
        label=_("Authenticator small name"),
        help_text=("Метка домена аутентификации"),
        required=False,
    )


    def validate(self, attrs):
        username = attrs.get('username', None)
        password = attrs.get('password', None)
        auth_id = attrs.get('authId', None)
        auth_small_name = attrs.get('authSmallName', None)
        auth_name = attrs.get('auth', None)

        if password == '':
            password = 'xdaf44tgas4xd5ñasdłe4g€@#½|«ð2'  # Extrange password if credential leaved empty
            attrs['password'] = password

        if username is None:
            attrs['username'] = None

        if password is None:
            attrs['password'] = None

        if auth_name == 'admin' or auth_small_name == 'admin':
            attrs['is_admin'] = True
            attrs['auth_obj'] = 'admin'
        else:
            attrs['is_admin'] = False
            try:
                # Will raise an exception if no auth found
                if auth_id is not None:
                    auth_obj = Authenticator.objects.get(uuid=processUuid(auth_id))
                elif auth_name is not None:
                    auth_obj = Authenticator.objects.get(name=auth_name)
                elif auth_small_name is not None:
                    auth_obj = Authenticator.objects.get(small_name=auth_small_name)
                else:
                    auth_obj = None
                attrs['auth_obj'] = auth_obj
            except Authenticator.DoesNotExist:
                msg = _('Accept invalid authenticator ID')
                raise serializers.ValidationError(msg, code='authorization')

        return attrs


class LegacyLoginErrorSerializer(serializers.Serializer):
    result = serializers.CharField(
        label=_("Response status"),
        help_text=("Статус ответа"),
    )
    error = serializers.CharField(
        label=_("Error"),
        help_text=("Описание ошибки"),
    )


class LegacyLoginSuccessSerializer(serializers.Serializer):
    result = serializers.CharField(
        label=_("Response status"),
        help_text=("Статус ответа"),
    )
    token = serializers.CharField(
        label=_("Token"),
        help_text=("Аутентификационный токен"),
    )
