'''
Created on 10 июн. 2022 г.

@author: dvd
'''
from rest_framework import serializers
from rest_framework.exceptions import PermissionDenied
from django.utils.translation import gettext_lazy as _
from django.contrib.sessions.backends.db import SessionStore

AUTH_TOKEN_HEADER = 'HTTP_X_AUTH_TOKEN'

class LegacyLogoutSerializer(serializers.Serializer):
    """ Сериалайзер для get и post запросов клиента на логаут. """
    result = serializers.CharField(
        label=_("Response status"),
        help_text=("Статус ответа"),
        read_only=True
    )

    def validate(self, attrs):
        request = self.context["request"]
        authToken = request.META.get(AUTH_TOKEN_HEADER)
        if authToken is None:
            # Не указан токен аутентификации
            raise PermissionDenied()
        if not SessionStore().exists(authToken):
            # Токена нет в базе
            raise PermissionDenied()
        session = SessionStore(session_key=authToken)
        attrs['session'] = session
        return attrs
