import datetime
import logging

from rest_framework.authtoken.models import Token
from rest_framework.response import Response
from rest_framework.generics import GenericAPIView
from ..schemas import LegacyLoginSchema
from ..serializers.legacy_login import LegacyLoginRequestSerializer
from django import http
from django.utils.translation import get_language
from uds.core.util.Config import GlobalConfig
from uds.core.util.stats import events
from uds.core.auths.auth import authenticate, authLogLogin
from uds.REST.handlers import Handler

logger = logging.getLogger(__name__)


class LegacyLogin(GenericAPIView):
    """
       post:
       Аутентификация пользователя на базе аутентификатора, логина и пароля.\n
       **Внимание!** При отправке запроса необходимо обязательно указать один из параметров:
       - authId
       - auth
       - authSmallName
    """
    serializer_class = LegacyLoginRequestSerializer
    schema = LegacyLoginSchema()

    def post(self, request, *args, **kwargs):

        serializer = self.serializer_class(data=request.data,
                                           context={'request': request})

        serializer.is_valid(raise_exception=True)
        is_admin = serializer.validated_data['is_admin']
        username = serializer.validated_data['username']
        password = serializer.validated_data['password']
        auth_obj = serializer.validated_data['auth_obj']
        locale = get_language()

        if username and password and auth_obj:
            if is_admin:
                if (GlobalConfig.SUPER_USER_LOGIN.get(True) == username and
                        GlobalConfig.SUPER_USER_PASS.get(True) == password):
                    token = Handler.genAuthToken(self, -1, username, locale, True, True),
                    return auth_success_response(token)

                else:
                    return auth_fail_response('Invalid Credentials')

            else:
                user = authenticate(username, password, auth_obj)
                if not user:
                    return auth_fail_response('Invalid Credentials', request, auth_obj, username)

                token = Handler.genAuthToken(self, auth_obj.id, user.name, locale, user.is_admin, user.staff_member)

            # Add an login event
            events.addEvent(auth_obj, events.ET_LOGIN,
                            username=username, srcip=request)
            events.addEvent(auth_obj, events.ET_PLATFORM,
                            platform=request.os.OS,
                            browser=request.os.Browser,
                            version=request.os.Version)

            return auth_success_response(token, request, auth_obj, username)

        elif auth_obj is None:
            return auth_fail_response('Invalid parameters (no auth)', request, auth_obj, username)
            
        else:
            return auth_fail_response('Invalid Credentials', request, auth_obj, username)


def auth_fail_response(error, request=None, authenticator=None, username=None):
    if request and authenticator and username:
        authLogLogin(request, authenticator, username, error)
    return Response({
        'error': error,
        'result': 'error',
    })


def auth_success_response(token, request=None, authenticator=None, username=None):
    if request and authenticator and username:
        authLogLogin(request, authenticator, username)
    return Response({
        'token': token,
        'result': 'ok'
    })
