from rest_framework import serializers


class VersionsSerializer(serializers.Serializer):
    version = serializers.CharField(
        required=False, allow_blank=True,
        help_text="Версия")
    url = serializers.CharField(help_text="Базовый URL для версии")


class SchemaSerializer(serializers.Serializer):
    version = serializers.CharField(help_text="Текущая версия схемы документа")
    versions = VersionsSerializer(
        required=True, many=True,
        help_text="Список доступных версий")


class ServerSerializer(serializers.Serializer):
    version = serializers.CharField(required=True, help_text="Версия сервера")


class ModulesSerializer(serializers.Serializer):
    id = serializers.CharField(
        required=False, allow_null=True,
        help_text="Идентификатор модуля REST API")
    versions = VersionsSerializer(
        required=False, many=True,
        help_text="Доступные версии")


class ApiSerializer(serializers.Serializer):
    schema = SchemaSerializer(
        required=True,
        help_text="Информация о схеме текущего документа")
    server = ServerSerializer(
        required=True,
        help_text="Информация о сервере")
    modules = ModulesSerializer(
        required=True, many=True,
        help_text="Список модулей REST API")
