'''
Created on 2 мая 2022 г.

@author: dvd
'''

from rest_framework.fields import SerializerMethodField


class TypedSerializerMethodField(SerializerMethodField):
    """
    Вариант поля :py:class:`rest_framework.fields.SerializerMethodField` с указанием типа данных
    для облегчения генерации спецификации OpenAPI.
    Требует переопределения метода :py:meth:`rest_framework.schemas.openapi.AutoSchema.map_field`.

    Пример использования:

    class ExampleSerializer(self):
        extra_info = TypedSerializerMethodField(field_type=serializers.BooleanField)

        def get_extra_info(self, obj):
            return ...  # Подготовка данных.

    class ExampleSchema(AutoSchema):
        def map_field(self, field):
            if isinstance(field, TypedSerializerMethodField):
                return AutoSchema.map_field(self, field.field_type())

            return AutoSchema.map_field(self, field)
    """

    def __init__(self, field_type=SerializerMethodField, **kwargs):
        self.field_type = field_type
        super().__init__(**kwargs)
