from django.conf.urls import url
from django.urls import include, reverse_lazy
from django.views.generic import TemplateView, RedirectView

from termidesk.api.agent import urls as agent_urls
from termidesk.api.auth import urls as auth_urls
from termidesk.api.discover import urls as discover_urls
from termidesk.api.external import urls as external_urls
from termidesk.api.test import urls as test_urls
from termidesk.api.permissions import NAME_DOCUMENTATION
from termidesk.api.swagger_schema import OpenApiSchemaView
from termidesk.api.webui import urls as webui_urls
from termidesk.api.wsproxy import urls as wsproxy_urls

import config

docs_path = r'^{}/'.format(NAME_DOCUMENTATION)

app_name = 'api'


modules_urlpatterns = [
    url(r'^agent/', include(agent_urls, namespace='agent')),
    url(r'^webui/', include(webui_urls, namespace='webui')),
    url(r'^auth/', include(auth_urls, namespace='auth')),
    url(r'^external/rubackup/', include(external_urls.rubackup_urls, namespace='external/rubackup')),
    url(r'^wsproxy/', include(wsproxy_urls, namespace='wsproxy')),
    url(r'^discover/', include(discover_urls, namespace='discover')),
]


schema_modules = [
    (urlpattern.namespace, list(sorted(urlpattern.namespace_dict, reverse=True)))
    for urlpattern in modules_urlpatterns
]


urlpatterns = [
    *modules_urlpatterns,
    url(
        r'^schema/openapi/(?P<module>.*)/(?P<submodule>[^/]*)',
        OpenApiSchemaView.as_view(),
        name='schema-openapi',
    ),
    url(
        docs_path + r'(?P<module>.+)$',
        TemplateView.as_view(
            template_name='termidesk/api/swagger-ui.html',
            extra_context={
                'schema_url': 'api:schema-openapi',
                'modules': schema_modules,
            },
        ),
        name='swagger-ui',
    ),
    url(
        docs_path + '$',
        RedirectView.as_view(
            url=reverse_lazy(
                'api:swagger-ui',
                kwargs={'module': schema_modules[0][0]},
            ),
        ),
    ),
]

if config.DJANGO_DEBUG:
    urlpatterns.append(url(r'^test/', include(test_urls, namespace='test')))
