from rest_framework import serializers
from uds.core.ui import gui as GUI

import logging

logger = logging.getLogger(__name__)


class GuiFieldsSerializer(serializers.Serializer):
    def to_representation(self, instance):
        """
        :param dict instance: Cловарь со спецификацией gui поля
        """

        fld_type = instance['gui']['type']
        types_for_char_gui = (GUI.InputField.TEXT_TYPE, GUI.NumericField.NUMERIC_TYPE,
                              GUI.InputField.INFO_TYPE, 'taglist')
        if fld_type in types_for_char_gui:
            field = GUICharFieldSerializer(instance)
            return field.data

        types_for_choice_gui = (GUI.ChoiceField.CHOICE_TYPE, GUI.MultiChoiceField.MULTI_CHOICE_TYPE)
        if fld_type in types_for_choice_gui:
            field = GUIChoiceFieldSerializer(instance)
            return field.data

        if fld_type == GUI.HiddenField.HIDDEN_TYPE:
            field = GUIHiddenFieldSerializer(instance)
            return field.data

        if fld_type == 'password':
            field = GUICharFieldSerializer(instance)
            return field.data

        if fld_type == 'checkbox':
            field = GUICharFieldSerializer(instance)
            return field.data

        raise ValueError("Unsupported field type {0}".format(fld_type))


class FieldDetailSerializer(serializers.Serializer):
    required = serializers.BooleanField()
    defvalue = serializers.CharField(allow_blank=True)
    value = serializers.CharField(allow_blank=True)
    minValue = serializers.CharField(required=False)
    maxValue = serializers.CharField(required=False)
    label = serializers.CharField(allow_blank=True)
    length = serializers.IntegerField()
    multiline = serializers.IntegerField(required=False)
    tooltip = serializers.CharField(allow_blank=True)
    rdonly = serializers.BooleanField()
    type = serializers.CharField()
    order = serializers.IntegerField()
    values = serializers.ListField(required=False)
    tab = serializers.CharField(required=False)


class FieldHiddenSerializer(serializers.Serializer):
    required = serializers.BooleanField()
    defvalue = serializers.CharField(allow_blank=True)
    value = serializers.CharField(allow_blank=True)
    label = serializers.CharField(allow_blank=True)
    length = serializers.IntegerField()
    tooltip = serializers.CharField(allow_blank=True)
    rdonly = serializers.BooleanField()
    type = serializers.CharField()
    order = serializers.IntegerField()


class FillsSerializer(serializers.Serializer):
    """ Сериализатор для указания callback функций обработки. """
    callbackName = serializers.CharField(allow_blank=True)
    parameters = serializers.ListField()


class FieldChoiceSerializer(serializers.Serializer):
    required = serializers.BooleanField()
    defvalue = serializers.CharField(allow_blank=True)
    value = serializers.CharField(allow_blank=True)
    label = serializers.CharField(allow_blank=True)
    length = serializers.IntegerField()
    tooltip = serializers.CharField(allow_blank=True)
    rdonly = serializers.BooleanField()
    type = serializers.CharField()
    order = serializers.IntegerField()
    values = serializers.ListField(required=False)
    fills = FillsSerializer(required=False)


class GUICharFieldSerializer(serializers.Serializer):
    """ Сериализатор для представления основных типов полей GUI. """
    name = serializers.CharField()
    value = serializers.CharField(allow_blank=True)
    gui = FieldDetailSerializer()


class GUIChoiceFieldSerializer(serializers.Serializer):
    """ Сериализатор для представления основных типов полей GUI. """
    name = serializers.CharField()
    value = serializers.CharField(allow_blank=True)
    gui = FieldChoiceSerializer()


class FieldBoolDetailSerializer(FieldDetailSerializer):
    defvalue = serializers.BooleanField()
    value = serializers.BooleanField()


class GUIBoolFieldSerializer(serializers.Serializer):
    """ Сериализатор для представления булевого типа полей GUI. """
    name = serializers.CharField()
    value = serializers.CharField(allow_blank=True)
    gui = FieldBoolDetailSerializer()


class GUIHiddenFieldSerializer(serializers.Serializer):
    """ Сериализатор для представления скрытых полей GUI. """
    name = serializers.CharField()
    value = serializers.CharField(allow_blank=True)
    gui = FieldHiddenSerializer()
