from rest_framework import serializers
from uds.core.managers.UserServiceManager import (MESSAGE_WARNING,
                                                  MESSAGE_INFORMATIONAL,
                                                  MESSAGE_ERROR)

LEVEL_VALUES = [MESSAGE_WARNING, MESSAGE_INFORMATIONAL, MESSAGE_ERROR]


class SendMessageSerializer(serializers.Serializer):
    """ Сериализатор получения пользователей фонда рабочих мест. """
    message = serializers.CharField(required=True, read_only=False)
    level = serializers.ChoiceField(required=True, choices=LEVEL_VALUES, read_only=False)

    class Meta:
        fields = ["message", "level"]

    def __init__(self, instance=None, data='', **kwargs):
        super().__init__(instance=instance, data=data, **kwargs)

    def validate_message(self, value):
        # Длина сообщения 1000 символов
        if len(value) > 1000:
            raise serializers.ValidationError("Длина сообщения >1000 символов")
        return value

    def validate_level(self, value):
        # Level сообщения строго обьявлен
        if value not in LEVEL_VALUES:
            raise serializers.ValidationError("Уровень сообщения некорректен. \
                Используйте 'warning', 'error' либо 'informational'")
        return value
