import logging
import uuid as uuid_type

from django.shortcuts import get_object_or_404
from rest_framework.exceptions import ValidationError
from rest_framework.generics import ListAPIView
from rest_framework.response import Response
from rest_framework.viewsets import ViewSet
from termidesk.api.permissions import AccessPermission, EnabledDeviceAuthRequiredPermission
from termidesk.api.webui.serializers.access_device import AccessDeviceSerializer
from termidesk.models.AccessDevice import ALLOWED_ACCESS, DENIED_ACCESS, AccessDevice
from uds.REST.model import OK

logger = logging.getLogger(__name__)


class AccessDeviceView(ListAPIView):
    """
    Список всех доступных устройств доступа.
    """
    model = AccessDevice
    serializer_class = AccessDeviceSerializer
    permission_classes = [AccessPermission, EnabledDeviceAuthRequiredPermission]

    def get_queryset(self):
        return self.model.objects.all()


class AccessDeviceStateView(ViewSet):
    """ Класс для смены статусов устройствам доступа. """
    permission_classes = [AccessPermission, EnabledDeviceAuthRequiredPermission]

    def change_status(self, request, uuid: uuid_type, status: str):
        """ Смена статуса устройства доступа. """
        device = get_object_or_404(AccessDevice, uuid=uuid)

        if device.access == status:
            raise ValidationError('Устройству доступа уже установлен этот статус.')

        if status not in (ALLOWED_ACCESS, DENIED_ACCESS):
            raise ValidationError('Передан недопустимый статус для установки: {}'.format(status))
        device.access = status
        device.save()

        return Response(OK)
