# -*- encoding: utf-8 -*-

from termidesk.models.UserPermission import NETWORK_VIEW, NETWORK_CREATE, NETWORK_CHANGE, NETWORK_DELETE
from ..serializers.network import NetworkSerializer

from uds.models import Network
from termidesk.api.webui.views.common_viewset import CrudCommonViewSet

from ...decorators import has_role_permission


class Networks(CrudCommonViewSet):
    """
    networks_overview:
        Список сетей
    networks_gui:
        Спецификация полей GUI для сетей
    create_network:
        Создание сети
    get_network:
        Получить информацию о сети
    update_network:
        Обновить сеть
    delete_network:
        Удалить сеть
    """
    lookup_field = 'uuid'
    model = Network
    serializer_class = NetworkSerializer

    @has_role_permission(NETWORK_VIEW)
    def get_network(self, request, *args, **kwargs):
        return super().retrieve(request, *args, **kwargs)

    @has_role_permission(NETWORK_VIEW)
    def networks_overview(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)

    @has_role_permission(NETWORK_CREATE)
    def create_network(self, request, *args, **kwargs):
        return super().create(request, *args, **kwargs)

    @has_role_permission(NETWORK_CHANGE)
    def update_network(self, request, *args, **kwargs):
        return super().update(request, *args, **kwargs)

    @has_role_permission(NETWORK_DELETE)
    def delete_network(self, request, *args, **kwargs):
        return super().destroy(request, *args, **kwargs)
