import logging

from django.shortcuts import get_object_or_404
from rest_framework import generics
from termidesk.api.permissions import AccessPermission
from termidesk.api.webui.helpers import GuiHelper
from termidesk.api.webui.schemas import GuiFieldsSchema
from termidesk.api.webui.serializers.gui import GuiFieldsSerializer
from uds.core.Environment import Environment
from uds.models.Provider import Provider

logger = logging.getLogger(__name__)


class ServicesGui(generics.ListAPIView):
    """Спецификация полей GUI для шаблонов рабочих мест."""
    permission_classes = [AccessPermission, ]
    serializer_class = GuiFieldsSerializer
    schema = GuiFieldsSchema(operation_id_base='WebuiServicesGuiFields')

    def get_queryset(self):
        service_type_name = self.kwargs.get("data_type", "")
        provider_uuid = self.kwargs.get("provider_uuid", "")
        provider = get_object_or_404(Provider, uuid=provider_uuid)
        parent_instance = provider.getInstance()
        service_type = parent_instance.getServiceByType(service_type_name)
        service = service_type(Environment.getTempEnv(), parent_instance)
        fields = GuiHelper.add_default_fields(service.guiDescription(service), ['name', 'comments', 'tags'])

        return sorted(fields, key=lambda f: f['gui']['order'])
