import logging

from rest_framework import generics
from termidesk.api.permissions import AccessPermission
from termidesk.api.webui.schemas import GuiFieldsSchema
from termidesk.api.webui.serializers.gui import GuiFieldsSerializer

from uds.core.managers.UserServiceManager import MESSAGE_LEVELS, MESSAGE_WARNING
from uds.core.ui import gui

logger = logging.getLogger(__name__)


class SendMessageAssignedUserServiceGui(generics.ListAPIView):
    """Спецификация полей GUI для политики конкретного типа."""
    permission_classes = [AccessPermission]
    serializer_class = GuiFieldsSerializer
    schema = GuiFieldsSchema(operation_id_base='WebuiMessageGuiFields')

    def get_queryset(self):
        fields = {
            "message": gui.TextField(
                    label='Сообщение',
                    order=1,
                    tooltip="Текст сообщения",
                    required=True,
                    length=1000,
                    multiline=4,
                ),
            "level": gui.ChoiceField(
                    label='Уровень',
                    order=2,
                    tooltip="Уровень сообщения",
                    required=True,
                    values=[{"id": lvl, "text": lvl} for lvl in MESSAGE_LEVELS],
                    defvalue=MESSAGE_WARNING
                ),
            }

        gui_spec = []
        for key, val in fields.items():
            # По аналогии с UserInterface.guiDescription
            gui_spec.append({'name': key, 'gui': val.guiDescription(), 'value': ''})

        return sorted(gui_spec, key=lambda f: f['gui']['order'])
