# -*- encoding: utf-8 -*-
'''
Created on 2 апр. 2022 г.

@author: dvd
'''
import logging
from itertools import filterfalse as exclude

from rest_framework import generics
from rest_framework.schemas.openapi import AutoSchema
from termidesk.api.permissions import AccessPermission

from termidesk.models.UserPermission import TRANSPORT_VIEW
from termidesk.transports.loudplay import transport_exclude_filter as experimental_loudplay_transports
from uds.core.transports import factory
from ...decorators import has_role_permission
from ..serializers.transport import TransportTypesSerializer

logger = logging.getLogger(__name__)


class TransportTypesView(generics.ListAPIView):
    """Список доступных типов протоколов доставки"""
    permission_classes = [AccessPermission]
    serializer_class = TransportTypesSerializer
    schema = AutoSchema(operation_id_base='WebuiTransportTypes')

    def get_queryset(self):
        transports_list = []
        transports = list(factory().providers().values())

        exclude_filters = [
            # Реинициализируем фильтр при каждом запросе для актуальности
            experimental_loudplay_transports()
        ]

        # Исключаем из списка протоколы, согласно разным правилам (фильтрам)
        for fltr in exclude_filters:
            transports = exclude(fltr, transports)

        for transport in transports:
            item = {
                'type': transport.type(),
                'icon': transport.icon(),
                'name': transport.name(),
                'group': transport.group,
                'description': transport.description()
            }
            transports_list.append(item)
        # Для удобства отображения в интерфейсе список отсортирован по названию протокола
        sorted_transports_list = sorted(transports_list, key=lambda _transport: _transport['name'])
        return sorted_transports_list

    @has_role_permission(TRANSPORT_VIEW)
    def get(self, request, *args, **kwargs):
        """
        Спецификация доступных типов протоколов доставки
        """

        return super().get(request, *args, **kwargs)
