import logging

from rest_framework.exceptions import APIException
from rest_framework.response import Response
from rest_framework.schemas.openapi import AutoSchema
from rest_framework.viewsets import ViewSet
from termidesk.api.wsproxy.v1.serializers import TokenSerializer
from termidesk.core.util import wsptoken

logger = logging.getLogger(__name__)


PATH_TO_RSA_PUB = 'private/wsproxy/id_rsa.pub'

# В данном API используется неканоничное применение операции POST.
# Для генерации более точной документации необходимы корректировки схемы openAPI.
class VerifySchema(AutoSchema):

    def get_path_parameters(self, path, method):
        parameters = AutoSchema.get_path_parameters(self, path, method)

        for param in parameters:
            if param["name"] == "token":
                param["description"] = "Токен клиента для валидации"

        return parameters

    def get_request_body(self, path, method):
        return {}
    
    def get_responses(self, path, method):
        responses = AutoSchema.get_responses(self, path, method)
        
        # Исправляем HTTP код возврата на реальный
        responses['200'] = responses["201"]
        del(responses['201'])

        return responses

class Verify(ViewSet):
    """
        post:
        Валидация токена вебсокет прокси для подключения на сервер Термидеск.  
        В случае успеха возвращает параметры целевого хоста для подключения через вебсокет прокси.
    """

    schema = VerifySchema(operation_id_base='WSProxyV1Verify')

    def get_serializer(self):
        return TokenSerializer()
    
    def post(self, request, *args, **kwargs):
        token = kwargs.get('token')

        with open(PATH_TO_RSA_PUB) as f:
            public_key = f.read()

        params = wsptoken.TokenDecoder(public_key).decode(token)
        if params is None:
            message = "Invalid token {}".format(token)
            logger.error(message)
            raise APIException(message)

        serializer = TokenSerializer(data=params)
        if serializer.is_valid() is False:
            logger.error('The token {} did not pass validation'.format(token))

        return Response(serializer.validated_data)
