# -*- coding: utf-8 -*-
"""
Данный модуль содержит определения событий аудита и вспомогательные классы.

Классы событий фактически являются контейнерами аттрибутов на базе именованых
кортежей.

Значения аттрибутов должны быть простыми типами (строка, целое и т.п.)
для упрощения поддержки сериализации.

Каждое событие содержит два обязательных аттрибута:
  * уникальное название
  * версия схемы события

В схему события входят:
  * названия аттрибутов
  * типы аттрибутов
  * количество аттрибутов
  * порядок аттрибутов

При изменении вышеперечисленного необходимо увеличивать версию схемы.
Для этого создается новое событие с тем же именем, но новым набором аттрибутов
и увеличенной версией схемы.

Например:
class WEB_USER_LOGIN_V1(
        BaseEventMixin,
        namedtuple(
            'WEB_USER_LOGIN_V1',
            ['authenticator', 'username', 'ip'])):
    __slots__ = ()
    _NAME = build_name('web.UserLogin')
    _VERSION = 1

class WEB_USER_LOGIN_V2(
        BaseEventMixin,
        namedtuple(
            'WEB_USER_LOGIN_V2',
            ['authenticator', 'username', 'srcip', 'user_agent'])):
    __slots__ = ()
    _NAME = build_name('web.UserLogin')
    _VERSION = 2

Классы событий предназначены для генерации событий аудита с помощью
механизма sys.audit python. Например:

import sys

evt = WEB_USER_LOGIN_V1(
        authenticator=auth_name,
        username=user.name,
        ip=request.ip).generate_event()
# или
evt = WEB_USER_LOGIN_V1(auth_name, request.user.name, request.ip).generate_event()

sys.audit(*evt)

Вызов generate_event() требуется для обхода ошибки в python 3.5, когда оператор
распаковки не вызывает переопределенный метод __iter__.

"""
from __future__ import unicode_literals

# В подмодулях пакета список экспортируемых модулей контролируется
# переменной __all__.
# См. также https://docs.python.org/3/tutorial/modules.html#importing-from-a-package
from .base import *
from .cli import *
from .policies import *
from .user import *
from .web import *

__author__ = 'Dmitry Dvoeglazov'
