
from __future__ import unicode_literals

import sys
import warnings
from itertools import chain


__all__ = (
    'EVENT_PREFIX',
    'build_name',
    'BaseEventMixin',
)
# Общий префикс для всех событий Termidesk
EVENT_PREFIX = 'termidesk.audit.events'


def build_name(name):
    """
    Вспомогательная функция для создания уникального имени события
    """
    return '.'.join([EVENT_PREFIX, name])


class BaseEventMixin:
    """
    Вспомогательный класс для добавления общей для всех событий функциональности.
    """

    __slots__ = ()

    def __iter__(self):
        """
        Название события и версия схемы должны идти первыми при перечислении
        элементов кортежа.
        """
        return chain(
            [self.event, self.version],
            super().__iter__()
        )

    @property
    def event(self):
        return self._NAME

    @property
    def version(self):
        return self._VERSION

    def generate_event(self):
        if sys.version_info > (3, 6):
            warn = (
                "Method generate_event deprecated for "
                "python > 3.6 and can be removed."
            )
            warnings.warn(warn, stacklevel=2)
        return iter(self)
