"""События, связанные с командной строкой"""

from __future__ import unicode_literals

from collections import namedtuple

from .base import BaseEventMixin, build_name


__all__ = (
    'CLI_SYSTEM_CONFIG_CHANGED_V1',
    'CLI_ENTITY_ACTION',
)


class CLI_SYSTEM_CONFIG_CHANGED_V1(
    BaseEventMixin,
    namedtuple(
        'CLI_SYSTEM_CONFIG_CHANGED_V1',
        ['username', 'section_name', 'parameter_key', 'parameter_value'])):
    """
        Событие: Изменение системных параметров Диспетчера подключений через командную строку
        Данные:
            :param username: Логин пользователя - args[0]
            :param section_name: Название секции - args[1]
            :param parameter_key: Название изменяемого параметра - args[2]
            :param parameter_value: Новое значение параметра - args[3]
    """
    __slots__ = ()
    _NAME = build_name('cli.SystemConfigChanged')
    _VERSION = 1
    DESCRIPTION_TEMPLATE = 'Пользователь "{args[0]}" изменил системный параметр ' \
                           '{args[1]}.{args[2]}={args[3]}'


class CLI_ENTITY_ACTION(
    BaseEventMixin,
    namedtuple(
        'CLI_ENTITY_ACTION_V1',
        ['username', 'entity', 'uuid', 'subtype', 'name', 'action'])):
    """
    Событие: CRUD операции с объектами через CLI.

       Данные:
       :param username: имя системного пользователя (который запустил команду) - args[0]
       :param entity: тип сущности - args[1]
       :param uuid: уникальный идентификатор - args[2]
       :param subtype: тип объекта - args[3]
       :param name: название объекта - args[4]
       :param action: действие над объектом (CRUD) - args[5]
    """
    _NAME = build_name('cli.EntityAction')
    __slots__ = ()
    _VERSION = 1
    DESCRIPTION_TEMPLATE = 'Пользователь "{args[0]}" выполнил операцию {args[5]} для объекта ' \
                           '{args[1]} ({args[2]}) {args[3]} "{args[4]}" '
