"""События, связанные с политиками"""

from __future__ import unicode_literals

from collections import namedtuple

from .base import BaseEventMixin, build_name


__all__ = (
    'POLICIES_GLOBAL_POLICY_CHANGED',
    'POLICIES_GLOBAL_POLICY_DELETED',
    'POLICIES_DEPLOYED_SERVICE_POLICY_CHANGED',
    'POLICIES_DEPLOYED_SERVICE_POLICY_DELETED',
)


class POLICIES_GLOBAL_POLICY_CHANGED(
    BaseEventMixin,
    namedtuple(
        'POLICIES_GLOBAL_POLICY_CHANGED',
        ['username', 'authenticator_name', 'policy_name', 'value', 'authenticator_uuid', 'value_raw'])):
    """Событие: Изменение глобальных политик.

       Данные:
       :param username: имя пользователя - args[0]
       :param authenticator_name: название домена аутентификации пользователя - args[1]
       :param policy_name: название политики - args[2]
       :param value: новое значение, в дружественном к пользователю описании - args[3]
       :param authenticator_uuid: id домена аутентификации пользователя - args[4]
       :param value_raw: новое значение, в оригинальном формате - args[5]
    """
    _NAME = build_name('policies.GlobalPolicyChanged')
    __slots__ = ()
    _VERSION = 1
    DESCRIPTION_TEMPLATE = 'Пользователь "{args[0]}({args[1]})" изменил значение' \
                           ' глобальной политики "{args[2]}" на "{args[3]}"'


class POLICIES_DEPLOYED_SERVICE_POLICY_CHANGED(
    BaseEventMixin,
    namedtuple(
        'POLICIES_DEPLOYED_SERVICE_POLICY_CHANGED',
        ['username', 'authenticator_name', 'policy_name', 'deployed_service_name', 'value',
         'authenticator_uuid', 'deployed_service_uuid', 'value_raw'])):
    """Событие: Изменение политик рабочего места.

       Данные:
       :param username: имя пользователя - args[0]
       :param authenticator_name: название домена аутентификации пользователя - args[1]
       :param policy_name: название политики - args[2]
       :param deployed_service_name: название фонда рабочих мест - args[3]
       :param value: новое значение, в дружественном к пользователю описании - args[4]
       :param authenticator_uuid: id домена аутентификации пользователя - args[5]
       :param deployed_service_uuid: id фонда рабочих мест - args[6]
       :param value_raw: новое значение, в оригинальном формате - args[7]
    """
    _NAME = build_name('policies.DeployedServicePolicyChanged')
    __slots__ = ()
    _VERSION = 1
    DESCRIPTION_TEMPLATE = 'Пользователь "{args[0]}({args[1]})" изменил' \
                           ' значение политики "{args[2]}" для фонда' \
                           ' "{args[3]}" на "{args[4]}"'


class POLICIES_DEPLOYED_SERVICE_POLICY_DELETED(
    BaseEventMixin,
    namedtuple(
        'POLICIES_DEPLOYED_SERVICE_POLICY_DELETED',
        ['username', 'authenticator_name', 'policy_name', 'deployed_service_name',
         'authenticator_uuid', 'deployed_service_uuid'])):
    """Событие: Сброс политики рабочего места.

       Данные:
       :param username: имя пользователя - args[0]
       :param authenticator_name: название домена аутентификации пользователя - args[1]
       :param policy_name: название политики - args[2]
       :param deployed_service_name: название фонда рабочих мест - args[3]
       :param authenticator_uuid: id домена аутентификации пользователя - args[5]
       :param deployed_service_uuid: id фонда рабочих мест - args[6]
    """
    _NAME = build_name('policies.DeployedServicePolicyDeleted')
    __slots__ = ()
    _VERSION = 1
    DESCRIPTION_TEMPLATE = 'Пользователь "{args[0]}({args[1]})" сбросил' \
                           ' значение политики "{args[2]}" для фонда' \
                           ' "{args[3]}"'


class POLICIES_GLOBAL_POLICY_DELETED(
    BaseEventMixin,
    namedtuple(
        'POLICIES_GLOBAL_POLICY_DELETED',
        ['username', 'authenticator_name', 'policy_name', 'authenticator_uuid'])):
    """Событие: Сброс глобальных политик.

       Данные:
       :param username: имя пользователя - args[0]
       :param authenticator_name: название домена аутентификации пользователя - args[1]
       :param policy_name: название политики - args[2]
       :param authenticator_uuid: id домена аутентификации пользователя - args[4]
    """
    _NAME = build_name('policies.GlobalPolicyDeleted')
    __slots__ = ()
    _VERSION = 1
    DESCRIPTION_TEMPLATE = 'Пользователь "{args[0]}({args[1]})" сбросил значение' \
                           ' глобальной политики "{args[2]}"'
