"""События, связанные с пользователем"""

from __future__ import unicode_literals

from collections import namedtuple

from termidesk.audit.events import BaseEventMixin, build_name


__all__ = (
    'USER_LOGGED_IN_V1',
    'USER_LOGGED_IN_V2',
    'USER_LOGGED_OUT_V1',
    'USER_LOGGED_OUT_V2',
    'USER_WORKPLACE_CONNECTION_REQUEST_V1',
    'USER_WORKPLACE_MESSAGE_SENT',
)


# DeprecationWarning: this event version will be removed in future [TSRV-749]
class USER_LOGGED_IN_V1(
    BaseEventMixin,
    namedtuple(
        'USER_LOGGED_IN_V1',
        ['authenticator', 'username', 'guest_os_username', 'ip', 'workplace', 'vm_name', ])):
    """
        Событие: Вход пользователя в ОС рабочего места
        Данные:
            :param authenticator: Название домена аутентификации пользователя - args[0]
            :param username: Логин пользователя - args[1]
            :param guest_os_username: Имя пользователя совершающего вход в гостевую ОС - args[2]
            :param ip: IP адрес, с которого был сделан запрос - args[3]
            :param workplace": имя фонда рабочих мест - args[4]
            :param vm_name": имя выданной машины - args[5]
    """
    __slots__ = ()
    _NAME = build_name('workplace.UserLogin')
    _VERSION = 1
    DESCRIPTION_TEMPLATE = 'Пользователь "{args[1]}({args[0]})" вошел в гостевую ОС ВМ {args[5]} ' \
                           'фонда {args[4]} как пользователь {args[2]} с ip-адреса {args[3]}'


class USER_LOGGED_IN_V2(
    BaseEventMixin,
    namedtuple(
        'USER_LOGGED_IN_V2',
        ['authenticator', 'username', 'guest_os_username', 'ip', 'workplace', 'vm_name', 'vm_ip'])):
    """
        Событие: Вход пользователя в ОС рабочего места
        Данные:
            :param authenticator: Название домена аутентификации пользователя - args[0]
            :param username: Логин пользователя - args[1]
            :param guest_os_username: Имя пользователя совершающего вход в гостевую ОС - args[2]
            :param ip: IP адрес, с которого был сделан запрос - args[3]
            :param workplace": имя фонда рабочих мест - args[4]
            :param vm_name": имя выданной машины - args[5]
            :param vm_ip: IP адрес выданной машины - args[6]
    """
    __slots__ = ()
    _NAME = build_name('workplace.UserLogin')
    _VERSION = 2
    DESCRIPTION_TEMPLATE = 'Пользователь "{args[1]}({args[0]})" вошел в гостевую ОС ВМ {args[5]}' \
                           '({args[6]}) фонда {args[4]} как пользователь {args[2]} с ip-адреса {args[3]}'


# DeprecationWarning: this event version will be removed in future [TSRV-749]
class USER_LOGGED_OUT_V1(
    BaseEventMixin,
    namedtuple(
        'USER_LOGGED_OUT_V1',
        ['authenticator', 'username', 'guest_os_username', 'ip', 'workplace', 'vm_name', ])):
    """
        Событие: Выход пользователя из ОС рабочего места
        Данные:
            :param authenticator: Название домена аутентификации пользователя - args[0]
            :param username: Логин пользователя - args[1]
            :param guest_os_username: Имя пользователя совершающего выход из гостевой ОС - args[2]
            :param ip: IP адрес, с которого был сделан запрос - args[3]
            :param workplace": имя фонда рабочих мест - args[4]
            :param vm_name": имя выданной машины - args[5]
    """
    __slots__ = ()
    _NAME = build_name('workplace.UserLogout')
    _VERSION = 1
    DESCRIPTION_TEMPLATE = 'Пользователь "{args[1]}({args[0]})" вышел из гостевой ОС ВМ {args[5]} ' \
                           'фонда {args[4]} как пользователь {args[2]} с ip-адреса {args[3]}'


class USER_LOGGED_OUT_V2(
    BaseEventMixin,
    namedtuple(
        'USER_LOGGED_OUT_V2',
        ['authenticator', 'username', 'guest_os_username', 'ip', 'workplace', 'vm_name', 'vm_ip'])):
    """
        Событие: Выход пользователя из ОС рабочего места
        Данные:
            :param authenticator: Название домена аутентификации пользователя - args[0]
            :param username: Логин пользователя - args[1]
            :param guest_os_username: Имя пользователя совершающего выход из гостевой ОС - args[2]
            :param ip: IP адрес, с которого был сделан запрос - args[3]
            :param workplace": имя фонда рабочих мест - args[4]
            :param vm_name": имя выданной машины - args[5]
            :param vm_ip: IP адрес выданной машины - args[6]
    """
    __slots__ = ()
    _NAME = build_name('workplace.UserLogout')
    _VERSION = 2
    DESCRIPTION_TEMPLATE = 'Пользователь "{args[1]}({args[0]})" вышел из гостевой ОС ВМ {args[5]}' \
                           '({args[6]}) фонда {args[4]} как пользователь {args[2]} с ip-адреса {args[3]}'


class USER_WORKPLACE_CONNECTION_REQUEST_V1(
    BaseEventMixin,
    namedtuple(
        'USER_WORKPLACE_CONNECTION_REQUEST_V1',
        ['authenticator', 'username', 'ip', 'workplace', 'vm_name', 'transport', ])):
    """
        Событие: Подключение пользователя к рабочему месту и начало работы
        Данные:
            :param authenticator: Название домена аутентификации пользователя - args[0]
            :param username: Логин пользователя - args[1]
            :param ip: IP адрес, с которого был сделан запрос - args[2]
            :param workplace: название фонда рабочих мест - args[3]
            :param vm_name": имя выданной машины - args[4]
            :param transport: название протокола доставки - args[5]
    """
    __slots__ = ()
    _NAME = build_name('user.WorkplaceConnectionRequest')
    _VERSION = 1
    DESCRIPTION_TEMPLATE = 'Пользователь "{args[1]}({args[0]})" подключился к ВМ {args[4]} ' \
                           'фонда {args[3]} по протоколу {args[5]} с ip-адреса {args[2]}'


class USER_WORKPLACE_MESSAGE_SENT(
    BaseEventMixin,
    namedtuple(
        'USER_WORKPLACE_MESSAGE_SENT',
        ['authenticator_name', 'authenticator_uuid', 'username',
         'deployed_service_name', 'deployed_service_uuid', 'user_service_name',
         'user_service_uuid', 'msg_level', 'msg_text'])):
    """Событие: Прекращение сессии пользователя по команде с сервера.

       Данные:
       :param authenticator_name: название домена аутентификации пользователя - args[0]
       :param authenticator_uuid: id домена аутентификации пользователя - args[1]
       :param username: имя пользователя - args[2]
       :param deployed_service_name: название фонда рабочих мест - args[3]
       :param deployed_service_uuid: id фонда рабочих мест - args[4]
       :param user_service_name: название рабочего места - args[5]
       :param user_service_uuid: id рабочего места - args[6]
       :param msg_level: тип сообщения - args[7]
       :param msg_text: текст сообщения - args[8]
    """
    _NAME = build_name('user.WorkplaceMessageSent')
    __slots__ = ()
    _VERSION = 1
    DESCRIPTION_TEMPLATE = 'Пользователь {args[2]}({args[0]}) отправил сообщение "{args[8]}" \
                            уровня {args[7]} на рабочее место {args[5]} фонда {args[3]}.'
