'''
Created on 10 авг. 2022 г.

@author: dvd
'''

from django.contrib.auth.models import Group as DjangoGroup

from termidesk.models.UserPermissionApi import get_api_user_permissions
from uds.core.ui.UserInterface import UserInterface, gui
from uds.core.util.State import State

from .util import USER_STATE_CHOICES


class BaseUsersUI(UserInterface):
    '''
    Базовый класс содержащий минимальной набор полей для создания/редактирования пользователей в
    доменах аутентификации.
    Обычно не должен использоваться напрямую, а наследоваться.
    '''

    __ui_version__ = 2  # GUI поля в дочерних классах наследуются автоматически

    def __init__(self, authenticator, values=None):
        super(BaseUsersUI, self).__init__(values)
        self._authenticator = authenticator

    name = gui.TextField(
        order=1,
        length=64,
        label='Логин',
        tooltip='Содержит логин пользователя',
        required=True,
        search=False,
    )

    real_name = gui.TextField(
        order=2,
        length=64,
        label='Имя',
        tooltip='Содержит имя пользователя',
        required=True,
    )

    comments = gui.TextField(
        order=3,
        label='Комментарий',
        length=256,
    )

    state = gui.ChoiceField(
        order=4,
        label='Статус',
        values=[gui.choiceItem(state, label) for state, label in USER_STATE_CHOICES],
        defvalue=State.ACTIVE,
        required=True
    )

    staff_member = gui.CheckBoxField(
        order=5,
        label='Персонал',
        defvalue=gui.FALSE
    )

    is_admin = gui.CheckBoxField(
        order=6,
        label='Администратор',
        defvalue=gui.FALSE
    )

    auth_groups = gui.MultiChoiceField(
        label='Группы',
        order=8,
        required=True
    )

    api_permissions = gui.MultiChoiceField(
        label='Разрешения доступа к API',
        order=9,
        required=False
    )

    roles = gui.MultiChoiceField(
        label='Роли',
        order=10,
        required=False
    )

    def initGui(self):
        '''
        Loads required values inside
        '''
        auth_instance = self._authenticator.getInstance()

        self.name.label = getattr(auth_instance, 'userNameLabel', self.name.label)

        groups = self._authenticator.groups.all().order_by('name')
        vals = []
        for g in groups:
            vals.append(gui.choiceItem(g.uuid, g.name))

        self.auth_groups.setValues(vals)

        permissions = [gui.choiceItem(permission.id, permission.name)
                       for permission in get_api_user_permissions()]
        self.api_permissions.setValues(permissions)

        roles = [
            gui.choiceItem(role.id, role.name) for role in DjangoGroup.objects.all().order_by("name")
        ]
        self.roles.setValues(roles)

        # Для предотвращения циклического импорта
        from uds.core.auths import Authenticator

        # Если метод отличается от родительского,
        # то поиск пользователей возможен
        # logger.debug('SearchUsers: {} neq {}  = {}'.format(
        #     auth_instance.__class__.searchUsers,
        #     Authenticator.searchUsers,
        #     (auth_instance.__class__.searchUsers != Authenticator.searchUsers)))
        if auth_instance.__class__.searchUsers != Authenticator.searchUsers:
            self.name.search = True
