'''
Created on 10 авг. 2022 г.

@author: dvd
'''
import warnings

from django.contrib.auth.models import Group as DjangoGroup

from termidesk.models.UserPermissionApi import get_api_user_permissions
from uds.core.ui.UserInterface import UserInterface, gui
from uds.core.util.State import State

from .util import USER_STATE_CHOICES


class LegacyUsersUI(UserInterface):
    '''
    Оригинальный класс для формирования спецификаций формы управления учетной
    записью пользователя в доменах аутентификации.
    Предполагает учет специфических особенностей разных типов доменов путем редактирования сформированной
    спецификации в методе guiDescription.
    От данного подхода решили отказаться в пользу специализированных классов UI для разных типов доменов.
    Данный класс объявлен устаревшим и не рекомендуется для дальнейшего использования и модификации.
    '''

    def __init__(self, parent, values=None):
        '''
        Do not forget to invoke this in your derived class using
        "super(self.__class__, self).__init__(environment, parent, values)".
        We want to use the env, parent methods outside class.
        If not called, you must implement your own methods
        cache and storage are "convenient" methods to access _env.cache and _env.storage
        '''
        super(LegacyUsersUI, self).__init__(values)
        self._parent = parent

        warnings.warn(
            ('LegacyUsersUI class is deprecated. '
             'Consider implementing specialized ui class for auth {type}'.format(type=parent.data_type)),
            DeprecationWarning,
            stacklevel=2
        )

    name = gui.TextField(
        order=1,
        length=64,
        label='Логин',
        tooltip='Содержит логин пользователя',
        required=True,
        search=False,
    )

    real_name = gui.TextField(
        order=2,
        length=64,
        label='Имя',
        tooltip='Содержит имя пользователя',
        required=True,
    )

    comments = gui.TextField(
        order=3,
        label='Комментарий',
        length=256,
    )

    state = gui.ChoiceField(
        order=4,
        label='Статус',
        values=[gui.choiceItem(state, label) for state, label in USER_STATE_CHOICES],
        defvalue=State.ACTIVE,
        required=True
    )

    staff_member = gui.CheckBoxField(
        order=5,
        label='Персонал',
        defvalue=gui.FALSE
    )

    is_admin = gui.CheckBoxField(
        order=6,
        label='Администратор',
        defvalue=gui.FALSE
    )

#     password = gui.PasswordField(
#         order=7,
#         length=32,
#         label='Пароль',
#         required=False
#     )

    auth_groups = gui.MultiChoiceField(
        label='Группы',
        order=8,
        # fills={
        #        'callbackName': 'userFillGroups',
        #        'function': UsersUIHelper.getResources,
        #        'parameters': ['id']
        # },
        required=True
    )

    api_permissions = gui.MultiChoiceField(
        label='Разрешения доступа к API',
        order=9,
        required=False
    )

    roles = gui.MultiChoiceField(
        label='Роли',
        order=10,
        required=False
    )

    # Override getGuiDescription method to handle password field
    # Т.к. некоторым аутентификаторам оно требуется, а некоторым нет
    @classmethod
    def guiDescription(cls, obj=None):
        # logger.debug("Inside overriden getGuiDesc")
        result = super(LegacyUsersUI, cls).guiDescription(obj)

        if obj is not None:
            pi = obj._parent.getInstance()
            need_pwd = pi.needsPassword
            # logger.debug("Need pwd: {}".format(need_pwd))

            if need_pwd:
                password = gui.PasswordField(
                    order=7,
                    length=32,
                    label='Пароль',
                    required=False
                )

                result.append({'name': 'password', 'gui': password.guiDescription(), 'value': ''})

        return result

    def initGui(self):
        '''
        Loads required values inside
        '''
        pi = self._parent.getInstance()

        try:
            self.name.label = pi.userNameLabel
        except AttributeError:
            pass

        groups = self._parent.groups.all().order_by('name')
        vals = []
        for g in groups:
            vals.append(gui.choiceItem(g.uuid, g.name))

        self.auth_groups.setValues(vals)

        permissions = (gui.choiceItem(permission.id, permission.name)
                       for permission in get_api_user_permissions())
        self.api_permissions.setValues(permissions)

        roles = (
            gui.choiceItem(role.id, role.name) for role in DjangoGroup.objects.all().order_by("name")
        )
        self.roles.setValues(roles)

        # Для предотвращения циклического импорта
        from uds.core.auths import Authenticator

        # Если метод отличается от родительского,
        # то поиск пользователей возможен
        # logger.debug('SearchUsers: {} neq {}  = {}'.format(
        #     pi.__class__.searchUsers,
        #     Authenticator.searchUsers,
        #     (pi.__class__.searchUsers != Authenticator.searchUsers)))
        if pi.__class__.searchUsers != Authenticator.searchUsers:
            self.name.search = True
