# -*- coding: utf-8 -*-
'''
Created on 13 дек. 2018 г.

@author: dvd
'''
from __future__ import unicode_literals

import pexpect
import logging

from functools import wraps
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import Permission
from django.http import HttpResponseRedirect, HttpResponseForbidden
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext as _
from guardian.models.models import GroupObjectPermission

from termidesk.models.UserPermission import VIEW_POLICY
from uds.core.util.Config import GlobalConfig


logger = logging.getLogger(__name__)


def root_pam_auth(password):
    '''Аутентификация суперпользователя (root) через PAM'''
    # Убираем возможные переносы каретки в пароле для подстраховки
    password = password.replace('\n', '')
    password = password.replace('\r', '')
    try:
        child = pexpect.spawn('/bin/su', ['-', 'root'], env={'LC_ALL': 'en_US.UTF-8'})
        child.expect('Password:')
        child.sendline(password)
        result = child.expect(['su: Authentication failure', 'root'])
        child.close()
    except Exception as err:
        child.close()
        logger.error("root_pam_auth Error authenticating. Reason: %s", err)
        return False
    if result == 0:
        logger.info("root_pam_auth Authentication failed for super user")
        return False
    else:
        logger.info("root_pam_auth Authentication succeeded for super user")
        return True


# Decorator to make easier protect pages that needs speci
def role_permission_required(permission: str):
    '''
    Декоратор позволяет проверить разрешения, установленные ролевой моделью
    '''
    def decorator(view_func):
        @wraps(view_func)
        def _wrapped_view(request, *args, **kwargs):
            '''
            Wrapped function for decorator
            '''
            if request.user is None:
                url = request.build_absolute_uri(GlobalConfig.LOGIN_URL.get())
                if GlobalConfig.REDIRECT_TO_HTTPS.getBool() is True:
                    url = url.replace('http://', 'https://')
                logger.debug('No user found, redirecting to {0}'.format(url))
                return HttpResponseRedirect(url)

            if not request.user.has_perm(permission):
                return HttpResponseForbidden(_('Forbidden'))

            return view_func(request, *args, **kwargs)
        return _wrapped_view
    return decorator


def role_object_permission_required(permission: str, model, name_uuid_field):
    """ Декоратор позволяет проверить разрешения к действиям над объектом, установленные ролевой моделью. """

    def decorator(view_func):
        @wraps(view_func)
        def _wrapped_view(request, *args, **kwargs):
            '''
            Wrapped function for decorator
            '''
            if request.user is None:
                url = request.build_absolute_uri(GlobalConfig.LOGIN_URL.get())
                if GlobalConfig.REDIRECT_TO_HTTPS.getBool() is True:
                    url = url.replace('http://', 'https://')
                logger.debug('No user found, redirecting to {0}'.format(url))
                return HttpResponseRedirect(url)

            uuid = kwargs.get(name_uuid_field)
            if not check_exists_permission_for_user(request.user, model, uuid, permission):
                return HttpResponseForbidden(_('Forbidden'))

            return view_func(request, *args, **kwargs)

        return _wrapped_view

    return decorator


def check_exists_permission_for_user(user, model, uuid, permission_codename):
    """ Проверка наличия разрешения к действиям над объектом. """
    if user.is_superuser:
        return True

    instance = get_object_or_404(model, uuid=uuid)
    content_type = ContentType.objects.get_for_model(model)
    id_groups = [group.id for group in user.roles.all()]
    permission = Permission.objects.get(codename=permission_codename)
    return GroupObjectPermission.objects.filter(
        object_pk=instance.id, content_type_id=content_type.id,
        permission_id=permission.id, group_id__in=id_groups).exists()


def user_has_view_policy_perm(func):
    """Проверка включенной политики и наличия прав администратора."""

    @wraps(func)
    def decorator(request, *args, **kwargs):

        if request.user.has_perm(VIEW_POLICY):
            return func(request, *args, **kwargs)

        return HttpResponseForbidden(_("Forbidden"))

    return decorator


def enabled_device_auth_required(func):
    """Проверка доступа к 'Устройства доступа'."""

    @wraps(func)
    def decorator(request, *args, **kwargs):
        device_auth_enabled = GlobalConfig.EXPERIMENTAL_DEVICE_AUTH_ENABLED.getBool(True)
        logger.debug('Checking device auth enabled: "{}"'.format(device_auth_enabled))
        if device_auth_enabled and request.user.is_admin:
            return func(request, *args, **kwargs)

        return HttpResponseForbidden(_("Forbidden"))

    return decorator
