from typing import Type
from termidesk.core.policies.BasePolicy import BasePolicy
from termidesk.models.PolicyGlobal import PolicyGlobal
from termidesk.models.PolicyDeployedService import PolicyDeployedService
from termidesk.core.util.singleton import SingletonMeta
from uds.core.Environment import Environment
from uds.models.ServicesPool import DeployedService


def get_global_policy(policy_cls: Type[BasePolicy]) -> BasePolicy:
    """Метод-помощник для получения значения глобальной политики"""

    policy = PolicyManager().get_global_policy(policy_cls)
    return policy


def get_deployed_service_policy(policy_cls: Type[BasePolicy],
                                deployed_service: DeployedService) -> BasePolicy:
    """Метод-помощник для получения значения политики для фонда рабочих мест"""

    policy = PolicyManager().get_deployed_service_policy(
        policy_cls,
        deployed_service.deployed_service,
    )
    return policy


class PolicyManager(metaclass=SingletonMeta):

    def get_global_policy(self, policy_type: BasePolicy):
        """
        Получение объекта глобальной политики по типу политики.

        :return: BasePolicy
        """
        try:
            model = PolicyGlobal.objects.get(data_type=policy_type.typeType)
            return model.getInstance()

        except PolicyGlobal.DoesNotExist:
            return policy_type(Environment.getGlobalEnv(), values={'value': policy_type.value.defValue})

    def get_deployed_service_policy(self, policy_type: BasePolicy, deployed_service: DeployedService):
        """
        Получение объекта политики фонда рабочих мест по типу политики и объекту фонда рабочих мест.

        :return: BasePolicy
        """
        try:
            model = PolicyDeployedService.get_policy(policy_type.typeType, deployed_service.id)
            return model.getInstance()
        except PolicyDeployedService.DoesNotExist:
            return self.get_global_policy(policy_type)
