from rest_framework.exceptions import APIException

from uds.core.ui.UserInterface import gui


class ChoicesPolicyUIMixin:
    """Миксин для политики с выбором значения из нескольких вариантов"""

    ENABLED = 1
    DISABLED = 0
    ALLOW_VALUES = {ENABLED: "Включен", DISABLED: "Выключен"}
    value = gui.ChoiceField(
        label="Значение",
        required=True,
        values=[],
        defvalue=ENABLED,
        order=2
    )

    @classmethod
    def guiDescription(cls, obj=None):
        """Дополнительно задает значения для выбора на основе ALLOW_VALUES"""
        cls.value.setValues([gui.choiceItem(key, value) for key, value in cls.ALLOW_VALUES.items()])
        return super().guiDescription(obj)

    @classmethod
    def get_view_value(cls, value):
        """Получение текстового представления значения, для отображения в интерфейсе."""

        try:
            value = int(value)
            result_value = cls.ALLOW_VALUES.get(value)
        except ValueError:
            result_value = None

        if result_value is None:
            values = list(cls.ALLOW_VALUES)
            raise APIException('Допустимы только значения: "{}"'.format(values))

        return result_value

    @classmethod
    def check_value(cls, value):
        cls.get_view_value(value)
