from typing import Dict, Type

from uds.core.util.UniqueNameGenerator import UniqueNameGenerator

from .FiniteUniqueNameGenerator import FiniteUniqueNameGenerator


_unique_name_generators = [
    UniqueNameGenerator,
    FiniteUniqueNameGenerator,
]


def get_all_unique_name_generators() -> Dict[str, Type[UniqueNameGenerator]]:
    """Возвращает словарь {имя: класс} генераторов уникальных имен для виртуальных машин"""

    return {
        generator.unique_id: generator for generator in _unique_name_generators
    }


def get_unique_name_generator(name):
    """Возвращает генератор уникальных имен для виртуальных машин,
    который установлен на данный момент в конфигурации
    """

    from uds.core.util.Config import GlobalConfig
    generator_id = GlobalConfig.GLOBAL_SERVICE_NAME_GENERATOR.get(force=True)
    generators = get_all_unique_name_generators()
    generator_cls = generators.get(generator_id, UniqueNameGenerator)
    return generator_cls(name)
