# -*- coding: utf-8 -*-
from __future__ import unicode_literals
'''
Created on 24 апр. 2022 г.

@author: dvd
'''
import argparse
import re
import sys
from django.core.management.base import CommandError
from django.db import transaction
from termidesk.core.util.utils import to_str
from termidesk.management.commands.common.subcommands.create import CreateSubcommand
from termidesk.management.commands.common.argtypes import positive_int_type
from uds.core import auths
from uds.models import Authenticator


def small_name_type(val):
    max_length = Authenticator._meta.get_field('small_name').max_length
    val = to_str(val)

    if len(val) > max_length:
        error = "Length exceeds {} chars".format(max_length)
        raise argparse.ArgumentTypeError(error)

    if re.search("(?u)[^ a-zA-Z0-9_]", val):
        error = "Allowed chars: a-z A-Z 0-9 _"
        raise argparse.ArgumentTypeError(error)

    return val


class CreateCommand(CreateSubcommand):
    name_of_service = 'домена аутентификации'

    types = [auth.typeType
             for auth in list(auths.factory().providers().values())]

    BASE_ARGS = ('name', 'small_name', 'priority', 'comments')

    _factory = None

    def add_subparser(self, subparsers):
        create_parser = subparsers.add_parser(
            CreateSubcommand.SUBCOMMAND,
            help="Создание домена аутентификации"
        )

        self.add_arguments_subcommand(create_parser)

        create_parser.add_argument(
            '--name', type=str,
            help="Имя домена аутентификации"
        )

        create_parser.add_argument(
            '--small_name', type=small_name_type,
            help=("Короткое имя домена аутентификации. "
                  "Допустимые символы: a-z A-Z 0-9 _. "
                  "Максимум 32 символа.")
        )

        create_parser.add_argument(
            '--priority', type=positive_int_type,
            help="Приоритет домена аутентификации. Значение по-умолчанию: 1.",
            default=1
        )

        create_parser.add_argument(
            '--comments', type=str, help="Комментарий", default=''
        )


    def handle_subcommand(self, options, stdout, stderr):
        if options['name'] is None:
            self._subcommand_parser.print_help()
            stdout.write(
                self.style.ERROR("Необходимо указать имя домена для создания")
            )
            sys.exit(1)
        
        if options['small_name'] is None:
            self._subcommand_parser.print_help()
            stdout.write(
                self.style.ERROR("Необходимо указать короткое имя домена для создания")
            )
            sys.exit(1)

        CreateSubcommand.handle_subcommand(self, options, stdout, stderr)

    @transaction.atomic()
    def create_service(self, auth_type, main_args, params):
        item = Authenticator.objects.create(**main_args)
        item.data_type = auth_type
        item.data = item.getInstance(params).serialize()
        item.save()

        from termidesk.rest.handlers import raise_cli_crud_event
        from termidesk.rest.handlers import CREATE_ACTION

        raise_cli_crud_event(
            entity=item.__class__.__name__,
            uuid=item.uuid,
            subtype=auth_type,
            name=item.name,
            action=CREATE_ACTION,
            username=None  # В качестве имени пользователя будет использоваться
                           # имя системного пользователя, запустившего cli-команду
        )

    def get_service_factory(self):
        if self._factory is None:
            self._factory = auths.factory()
        return self._factory

    def validate_base_data(self, data):
        """ Проверка наличия в БД. """
        try:
            Authenticator.objects.get(name=data['name'])
            error = ('Домен аутентификации {} '
                     'уже существует').format(data['name'])
            raise CommandError(error)
        except Authenticator.DoesNotExist:
            pass

        try:
            Authenticator.objects.get(small_name=data['small_name'])
            error = ('Домен аутентификации c коротким именем {} '
                     'уже существует').format(data['small_name'])
            raise CommandError(error)
        except Authenticator.DoesNotExist:
            pass
