# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import collections

from django.db.models import Count, F, Q
from django.utils.translation import gettext as _

from termidesk.core.managers.PolicyManager import PolicyManager
from termidesk.policies.ProtocolSelectionPolicy import ProtocolSelectionPolicy
from uds.core.util.State import State
from uds.models import DeployedService
from uds.models.ServicesPoolGroup import DEFAULT_GROUP_NAME

from termidesk.management.commands.common.subcommands.list import (
    ListSubcommand
)


class ListCommand(ListSubcommand):

    def add_subparser(self, subparsers):
        list_parser = subparsers.add_parser(
            self.SUBCOMMAND,
            help="Show items of a service"
        )
        self.add_arguments_subcommand(list_parser)

    @staticmethod
    def has_elements():
        return DeployedService.objects.exists()

    @staticmethod
    def get_table_records():
        """
        Получение необходимой структуры.
        """
        deployed_services = DeployedService.objects.annotate(
            workspaces=Count('userServices', filter=Q(userServices__state=State.USABLE)),
            prepare=Count('userServices', filter=Q(userServices__state=State.PREPARING)),
            group_name=F('servicesPoolGroup__name'),
            template_name=F('service__name'),
        )

        for service in deployed_services:
            show_transports = PolicyManager().get_deployed_service_policy(
                ProtocolSelectionPolicy, service
            ).is_allowed()

            yield collections.OrderedDict([
                ('id', service.uuid),
                ('name', service.name),
                ('status', _(State.toString(service.state))),
                ('workspaces', str(service.workspaces)),
                ('prepare', str(service.prepare)),
                ('protocolChoice', "Да" if show_transports else "Нет"),
                ('workspaceGroup', service.group_name or DEFAULT_GROUP_NAME),
                ('template', service.template_name),
            ])

    def show_elements(self, stdout, stderr):
        pattern = "{0:50}   {1:30}   {2:14}   {3:14}   {4:10}  {5:15}   {6:19}   {7:13}"

        stdout.write(
            self.style.SUCCESS("Список фондов рабочих мест:")
        )

        stdout.write(
            self.style.SUCCESS(
                pattern.format(
                    'ID',
                    'Название',
                    'Статус',
                    'Рабочие места',
                    'Готовятся',
                    'Выбор протокола',
                    'Группа рабочих мест',
                    'Шаблон'
                )
            )
        )

        for service in self.get_table_records():
            stdout.write(
                self.style.WARNING(
                    pattern.format(*(service[key] for key in service))
                )
            )

        from termidesk.rest.handlers import raise_cli_crud_event
        from termidesk.rest.handlers import READ_ACTION
        raise_cli_crud_event(
            entity=DeployedService.__name__,
            uuid='',
            subtype='',
            name='',
            action=READ_ACTION,
            username=None  # В качестве имени пользователя будет использоваться
                           # имя системного пользователя, запустившего cli-команду
        )
