from __future__ import unicode_literals

import logging

from rest_framework.generics import get_object_or_404
from server.celery import app as celery
from uds.core.jobs.DelayedTask import DelayedTask
from uds.core.util import log
from uds.models.UserService import UserService
from uds.core.managers.UserServiceManager import UserServiceManager

logger = logging.getLogger(__name__)

TAG = 'message-us-'
SUGGESTED_TIME = 10


class MessageUserService(DelayedTask):
    '''
    Отложенная задача, выполняющая отправку сообщения на рабочее место.
    '''
    celery_task_name = 'T.termidesk.notifications.delayed_task.message_userservice.MessageUserService'

    @celery.task(name=celery_task_name)
    def run_in_celery(self):
        self.execute()

    def __init__(self, user_service_uuid, message, level):
        super(MessageUserService, self).__init__()

        self._user_service_uuid = user_service_uuid
        self._message = message
        self._level = level

    def run(self):
        user_service = get_object_or_404(UserService, uuid=self._user_service_uuid)

        log_message = 'Sending message to workplace.'
        log.doLog(user_service, log.INFO, log_message, log.SERVICE)

        try:
            message_sent = UserServiceManager.manager().sendMessage(user_service, self._message, self._level)
            if message_sent:
                log_message = 'Message accepted.'
                log.doLog(user_service, log.INFO, log_message, log.SERVICE)
                return True
        except Exception as error:
            log_message = 'Error sending message: {}.'.format(error)
            log.doLog(user_service, log.ERROR, log_message, log.SERVICE)
            logger.error(log_message)
