from rest_framework.exceptions import APIException
from termidesk.core.policies.BasePolicy import BasePolicy
from uds.core.ui.UserInterface import gui


class DetachableUserProfileEnabledPolicy(BasePolicy):

    typeName = "Отделяемый пользовательский профиль"
    typeType = "DetachableUserProfileEnabledPolicy"
    typeDescription = (
        "Использование отделяемого пользовательского профиля на рабочих местах. \n"
        "Примечание: Политика применяется при старте рабочего места."
    )

    ENABLED = 1
    DISABLED = 0
    ALLOW_VALUES = {ENABLED: "Включен", DISABLED: "Выключен"}

    value = gui.ChoiceField(
        label="Значение",
        required=True,
        values=[gui.choiceItem(key, value) for key, value in ALLOW_VALUES.items()],
        defvalue=DISABLED,
        order=2
    )

    @classmethod
    def get_view_value(cls, value):
        """Получение текстового представления значения, для отображения в интерфейсе."""
        try:
            value = int(value)
        except ValueError:
            values = [allow_value for allow_value in cls.ALLOW_VALUES.keys()]
            raise APIException('Допустимы только значения: "{values}"'.format(values=values))
        return cls.ALLOW_VALUES.get(value)

    @classmethod
    def check_value(cls, value):
        if cls.get_view_value(value) is None:
            raise APIException('Передано не допустимое значение: "{value}"'.format(value=value))

    def is_allowed(self):
        """ Проверка разрешение использования отделяемого профиля. """
        return self.get_value() == str(self.ENABLED)
