# -*- coding: utf-8 -*-
'''
Created on 22 авг. 2018 г.

@author: dvd
'''
from __future__ import unicode_literals

from termidesk.models.UserPermission import AUTHENTICATOR_VIEW, TRANSPORT_VIEW
from uds.core import Module
from uds.core.ui.UserInterface import gui
from uds.REST import ResponseError
from uds.core.auths import Authenticator
from uds.core.util.request import getRequest
from uds.models import Authenticator as AuthenticatorModel
from uds.models import User, Group
from uds.models import Transport
import logging

logger = logging.getLogger(__name__)


def remove_field(_gui_description, _field):
    """Функция удаляет из интерфейса указанное поле"""
    indexes = [i for i, x in enumerate(_gui_description) if x['name'] == _field]
    if len(indexes) > 0:
        index = indexes[0]
        del _gui_description[index]

    return _gui_description


class UsersUIHelper(object):
      
    @staticmethod
    def getResources(parameters):
        logger.debug('Parameters received by getResources Helper: {0}'.format(parameters))
        
        data = [
            {
                'name': 'groups',
                'values': [{'id': 'g1', 'text': 'group1'}, {'id': 'g2', 'text': 'group2'}]
            }
        ]

        logger.debug('return data: {0}'.format(data))
        return data

        
class GroupUI(Module):
    
    def __init__(self, environment, parent, values=None):
        '''
        Do not forget to invoke this in your derived class using "super(self.__class__, self).__init__(environment, parent, values)".
        We want to use the env, parent methods outside class. If not called, you must implement your own methods
        cache and storage are "convenient" methods to access _env.cache and _env.storage
        '''
        super(GroupUI, self).__init__(environment, values)
        self._parent = parent
    
    name = gui.TextField(order=1,
        length=64,
        label='Имя группы',
        required=True,
        rdonly=True,
        search = False,
    )
    
    comments = gui.TextField(
        order = 3,
        label = 'Комментарий',
        length= 256,
    )
    
    state = gui.ChoiceField(
        order = 4,
        label = 'Статус',
        values=[
            gui.choiceItem('A', 'Активный'),
            gui.choiceItem('I', 'Неактивный'),
        ],
        defvalue='A',
        required=True
        
    )
    
    type = gui.HiddenField(order=10,
        length=64,
        required=True,
        defvalue='group'
    )
    
    def initGui(self):
        pi = self._parent.getInstance()
        
        try:
            self.name.label = pi.groupNameLabel
        except AttributeError:
            pass
        
        # Если метод отличается от родительского, 
        # то поиск групп возможен
        if pi.__class__.searchGroups != Authenticator.searchGroups:
            self.name.search = True

class MetaGroupUI(Module):
    
    def __init__(self, environment, parent, values=None):
        '''
        Do not forget to invoke this in your derived class using "super(self.__class__, self).__init__(environment, parent, values)".
        We want to use the env, parent methods outside class. If not called, you must implement your own methods
        cache and storage are "convenient" methods to access _env.cache and _env.storage
        '''
        super(MetaGroupUI, self).__init__(environment, values)
        self._parent = parent
    
    name = gui.TextField(order=1,
        length=64,
        label='Имя метагруппы',
        required=True,
        search = False,
    )
    
    meta_if_any = gui.CheckBoxField(
        order = 2,
        label = 'Тип совпадения',
        on_label = 'Любой',
        off_label = 'Все',
    )
    
    comments = gui.TextField(
        order = 3,
        label = 'Комментарий',
        length= 256,
    )
    
    state = gui.ChoiceField(
        order = 4,
        label = 'Статус',
        values=[
            gui.choiceItem('A', 'Активный'),
            gui.choiceItem('I', 'Неактивный'),
        ],
        defvalue='A',
        required=True
        
    )
    
    type = gui.HiddenField(order=10,
        length=64,
        required=True,
        defvalue='meta'
    )
    
    groups = gui.MultiChoiceField(
        label='Группы',
        order=8,
        required=False
    )
    
    def initGui(self):
        '''
        Loads required values inside
        '''
        
        pi = self._parent.getInstance()
        
        try:
            self.name.label = pi.groupNameLabel
        except AttributeError:
            pass
        
        groups = self._parent.groups.filter(is_meta=False).order_by('name')
        vals = []
        for g in groups:
            vals.append(gui.choiceItem(g.uuid, g.name))

        self.groups.setValues(vals)

class ServicePoolsGroupUIHelper(object):

    @staticmethod
    def __get_groups_from_domain(domain_uuid):
        request = getRequest()
        if request.user.has_perm(AUTHENTICATOR_VIEW):
            domain = AuthenticatorModel.objects.get(uuid=domain_uuid)
            groups = Group.objects.filter(manager=domain)
        else:
            # Если у пользователя нет разрешений, возвращается пустой queryset
            groups = Group.objects.none()

        return groups
      
    @staticmethod
    def getResources(parameters):
        groups = ServicePoolsGroupUIHelper.__get_groups_from_domain(parameters['domain'])
        values = []
        for g in groups:
            values.append({
                'id': g.uuid,
                'text': g.name,
            })
                           
        data = [
            {
                'name': 'id',
                'values': values,
            }
        ]

        return data

class ServicePoolsGroupUI(Module):
    
    def __init__(self, environment, parent, values=None):
        '''
        Do not forget to invoke this in your derived class using "super(self.__class__, self).__init__(environment, parent, values)".
        We want to use the env, parent methods outside class. If not called, you must implement your own methods
        cache and storage are "convenient" methods to access _env.cache and _env.storage
        '''
        super(ServicePoolsGroupUI, self).__init__(environment, values)
        self._parent = parent
    
    domain = gui.ChoiceField(
        label = 'Домен аутентификации',
        required=True,
        fills={
            'callbackName': 'domainFillGroups',
            'function': ServicePoolsGroupUIHelper.getResources,
            'parameters': ['domain']
        },
    )
    
    id = gui.ChoiceField(
        label = 'Группа',
        required=True
    )
    
    def initGui(self):
        domains = self.__get_domains()
        vals = []
        for domain in domains:
            vals.append(gui.choiceItem(domain.uuid, domain.name))

        self.domain.setValues(vals)

    @staticmethod
    def __get_domains():
        request = getRequest()
        if request.user.has_perm(AUTHENTICATOR_VIEW):
            domain = AuthenticatorModel.objects.all()
        else:
            # Если у пользователя нет разрешений, возвращается пустой queryset
            domain = AuthenticatorModel.objects.none()

        return domain


class ServicePoolsUserUIHelper(object):

    @staticmethod
    def __get_users_from_domain(domain_uuid):
        request = getRequest()
        if request.user.has_perm(AUTHENTICATOR_VIEW):
            domain = AuthenticatorModel.objects.get(uuid=domain_uuid)
            users = User.objects.filter(manager=domain)
        else:
            # Если у пользователя нет разрешений, возвращается пустой queryset
            users = User.objects.none()

        return users

    @staticmethod
    def getResources(parameters):
        users = ServicePoolsUserUIHelper.__get_users_from_domain(parameters['domain'])
        values = []
        for u in users:
            values.append({
                'id': u.uuid,
                'text': u.name,
            })

        data = [
            {
                'name': 'id',
                'values': values,
            }
        ]

        return data


class ServicePoolsUserUI(Module):

    def __init__(self, environment, parent, values=None):
        '''
        Do not forget to invoke this in your derived class using "super(self.__class__, self).__init__(environment, parent, values)".
        We want to use the env, parent methods outside class. If not called, you must implement your own methods
        cache and storage are "convenient" methods to access _env.cache and _env.storage
        '''
        super(ServicePoolsUserUI, self).__init__(environment, values)
        self._parent = parent

    domain = gui.ChoiceField(
        label='Домен аутентификации',
        required=True,
        fills={
            'callbackName': 'domainFillUsers',
            'function': ServicePoolsUserUIHelper.getResources,
            'parameters': ['domain']
        },
    )

    id = gui.ChoiceField(
        label='Пользователь',
        required=True
    )

    def initGui(self):
        domains = self.__get_domains()
        vals = []
        for domain in domains:
            vals.append(gui.choiceItem(domain.uuid, domain.name))

        self.domain.setValues(vals)

    @staticmethod
    def __get_domains():
        request = getRequest()
        if request.user.has_perm(AUTHENTICATOR_VIEW):
            domain = AuthenticatorModel.objects.all()
        else:
            # Если у пользователя нет разрешений, возвращается пустой queryset
            domain = AuthenticatorModel.objects.none()

        return domain


class ServicePoolsTransportUI(Module):
    
    def __init__(self, environment, parent, values=None):
        '''
        Do not forget to invoke this in your derived class using "super(self.__class__, self).__init__(environment, parent, values)".
        We want to use the env, parent methods outside class. If not called, you must implement your own methods
        cache and storage are "convenient" methods to access _env.cache and _env.storage
        '''
        super(ServicePoolsTransportUI, self).__init__(environment, values)
        self._parent = parent
    
    id = gui.ChoiceField(
        label = 'Протокол доставки',
        required=True
    )
    
    def initGui(self):
        
        transports = self.__get_transports()
        vals = []
        for transport in transports:
            vals.append(gui.choiceItem(transport.uuid, transport.name))

        self.id.setValues(vals)

    @staticmethod
    def __get_transports():
        request = getRequest()
        if request.user.has_perm(TRANSPORT_VIEW):
            transports = Transport.objects.all()
        else:
            # Если у пользователя нет разрешений, возвращается пустой queryset
            transports = Transport.objects.none()

        return transports

    @staticmethod
    def __get_transports():
        request = getRequest()
        if request.user.has_perm(TRANSPORT_VIEW):
            transports = Transport.objects.all()
        else:
            # Если у пользователя нет разрешений, возвращается пустой queryset
            transports = Transport.objects.none()

        return transports
