# -*- coding: utf-8 -*-
'''
Created on Nov 8, 2019

@author: dvd
'''
from __future__ import unicode_literals

import logging

from django.utils.translation import gettext_lazy as _

from uds.core import services
from uds.core.services import types as serviceTypes
from uds.core.ui.UserInterface import gui

from .OVirtStaticDeployment import OVirtStaticDeployment

logger = logging.getLogger(__name__)


class OVirtStaticService(services.Service):
    """Сервис для статичной ВМ oVirt/RHEV"""

    typeName = _('Статичная ВМ oVirt/RHEV')
    typeType = 'OVirtStaticService'
    typeDescription = _('Предоставляет доступ к статичным включенным ВМ по Spice протоколу')
    iconFile = 'service.png'

    maxDeployed = -1  # If the service provides more than 1 "provided service" (-1 = no limit, 0 = ???? (do not use it!!!), N = max number to deploy
    usesCache = False  # Cache are running machine awaiting to be assigned
    usesCache_L2 = False  # L2 Cache are running machines in suspended state
    needsManager = False  # If the service needs a s.o. manager (managers are related to agents provided by services itselfs, i.e. virtual machines with agent)
    mustAssignManually = False  # If true, the system can't do an automatic assignation of a deployed user service from this service

    deployedType = OVirtStaticDeployment

    servicesTypeProvided = (serviceTypes.VDI,)

    # Gui
    vmId = gui.ChoiceField(
        length=64,
        label=_("ВМ"),
        order=110,
        tooltip=_('Статическая ВМ для рабочего места'),
        required=True,
    )

    def getConsoleConnection(self, machineId):
        return self.parent().getConsoleConnection(machineId)

    def getHostname(self):
        return self.parent().host.value

    def getMachineState(self, machineId):
        '''
        Invokes getMachineState from parent provider
        (returns if machine is "active" or "inactive"

        Args:
            machineId: If of the machine to get state

        Returns:
            one of this values:
             unassigned, down, up, powering_up, powered_down,
             paused, migrating_from, migrating_to, unknown, not_responding,
             wait_for_launch, reboot_in_progress, saving_state, restoring_state,
             suspended, image_illegal, image_locked or powering_down
             Also can return'unknown' if Machine is not known
        '''
        return self.parent().getMachineState(machineId)

    def startMachine(self, machineId):
        '''
        Tries to start a machine. No check is done, it is simply requested to oVirt.

        This start also "resume" suspended/paused machines

        Args:
            machineId: Id of the machine

        Returns:
        '''
        return self.parent().startMachine(machineId)

    def initGui(self):
        """
        Подгружает необходимые значения для полей
        """

        machines = self.parent().getMachines()
        vals = []
        for machine in machines:
            vals.append(gui.choiceItem(machine['id'], machine['name']))
        self.vmId.setValues(vals)
