# -*- coding: utf-8 -*-

"""
@author: dvd
"""

from __future__ import unicode_literals

from django.utils.translation import gettext_noop as _
from django.urls import reverse
from django.http import HttpResponseRedirect

from uds.core.ui.UserInterface import gui
from uds.core.transports.BaseTransport import Transport
from uds.core.transports.BaseTransport import TUNNELED_GROUP

from uds.core.transports import protocols
from uds.core.util import OsDetector
from uds.models import TicketStore

from ..utils import WEBSOCKET_GATEWAY_URL_FORMAT
from ..utils import WEBSOCKET_GATEWAY_URL_DEFVAL
from ..utils import verify_gateway_url

import logging

logger = logging.getLogger(__name__)


class HTML5SPICETransport(Transport):
    """
    Provides access via RDP to service.
    This transport can use a domain. If username processed by authenticator contains '@',
    it will split it and left-@-part will be username, and right password
    """
    typeName = _('SPICE (HTML5, через локальный прокси)')
    typeType = 'HTML5SPICETransport'
    typeDescription = 'Протокол доставки на основе SPICE/HTML5 через встроенный вебсокет прокси сервер'
    iconFile = 'html5.png'

    ownLink = True
    supportedOss = OsDetector.allOss
    protocol = protocols.SPICE
    group = TUNNELED_GROUP

    gatewayUrl = gui.TextField(
        label='URL шлюза',
        order=1,
        defvalue=WEBSOCKET_GATEWAY_URL_DEFVAL,
        tooltip=_('Базовый URL вебсокет шлюза. (Формат: {fmt})'.format(fmt=WEBSOCKET_GATEWAY_URL_FORMAT)),
        required=True, tab=gui.TUNNEL_TAB)

    tunnelWait = gui.NumericField(
        length=3,
        label=_('Время ожидания соединения'),
        defvalue='20', minValue=1, maxValue=1800, order=2,
        tooltip=_('Максимальное время ожидание установления соединения'),
        required=True, tab=gui.TUNNEL_TAB)

    def isAvailableFor(self, userService, ip):
        """
        Checks if the transport is available for the requested destination ip
        Override this in yours transports
        """
        logger.debug('Checking availability for {0}'.format(ip))
        return True  # Spice is available, no matter what IP machine has (even if it does not have one)

    def getLink(self, userService, transport, ip, os, user, request):
        userServiceInstance = userService.getInstance()

        con = userServiceInstance.getConsoleConnection()

        logger.debug('Connection data: {}'.format(con))

        port, secure_port = con['port'], con['secure_port']
        port = -1 if port is None else port
        secure_port = -1 if secure_port is None else secure_port

        # Build params dict
        params = {
            'workplace_name': userService.deployed_service.name,
            'wsproxy': self.gatewayUrl.value,
            'host': con['address'],
            'port': port,
            'tls-port': secure_port,
            'spice_password': con['ticket']['value'],
            'tunnelWait': self.tunnelWait.num()
        }

        logger.debug('SPICE Params: {0}'.format(params))

        ticket = TicketStore.create(params)

        return HttpResponseRedirect(reverse('Html5SpiceClient', kwargs={'ticketId': ticket}))

    @staticmethod
    def test(env, data):
        ws_uri = data.get('gatewayUrl', '')
        result = verify_gateway_url(ws_uri)

        return result
