"""
Created on May 6, 2015
@author: dkmaster

Modifications
Copyright (c) 2017 Termilab, Dmitry Dvoeglazov
All rights reserved.
Added proxy option
@author: Dmitry Dvoeglazov
"""

from __future__ import unicode_literals


__updated__ = '2017-02-21'


TEMPLATE = '''\
[virt-viewer]
type={type}
host={host}
port={port}
password={password}
tls-port={tls_port}
fullscreen={fullscreen}
title={title}:%d - Press SHIFT+F12 to Release Cursor
enable-smartcard={smartcard}
enable-usb-autoshare={usb_auto_share}
delete-this-file={delete_file}
usb-filter=-1,-1,-1,-1,1
tls-ciphers=DEFAULT
host-subject={host_subject}
ca={ca}
toggle-fullscreen=shift+f11
release-cursor=shift+f12
secure-attention=ctrl+alt+end
{secure_channel}
{proxy}

[termidesk]
{termidesk_proxy}
{termidesk_clipboard}
{termidesk_filecopy}
{termidesk_pcsc}
'''


class RemoteViewerFile(object):
    type = 'spice'
    host = None
    port = None
    tls_port = None
    password = None
    title = 'Virtual Desktop'
    host_subject = ''
    ca = ''

    smartcard = False
    usb_auto_share = True

    delete_file = True

    def __init__(self, host, port, tls_port, password, ca, host_subject, fullscreen='', proxy='',
                 termidesk_proxy='', termidesk_clipboard='', termidesk_filecopy='', termidesk_pcsc=''):
        self.host = host
        self.port = port
        self.tls_port = tls_port
        self.password = password
        self.ca = ca
        self.host_subject = host_subject
        self.fullscreen = fullscreen
        self.proxy = proxy
        self.termidesk_proxy = termidesk_proxy
        self._termidesk_clipboard = termidesk_clipboard
        self._termidesk_filecopy = termidesk_filecopy
        self._termidesk_pcsc = termidesk_pcsc

    @property
    def as_file(self):
        return self.get()

    def get(self):
        smartcard = '01'[self.smartcard]
        delete_file = '01'[self.delete_file]
        usb_auto_share = '01'[self.usb_auto_share]

        # So we get '\\n' and script works fine after replacement
        ca = self.ca.strip().replace('\n', '\\\\n')

        proxy_option = ''
        if self.proxy:
            proxy_option = 'proxy={}'.format(self.proxy)

        termidesk_proxy_option = ''
        if self.termidesk_proxy:
            termidesk_proxy_option = 'proxy={}'.format(self.termidesk_proxy)

        return TEMPLATE.format(
            type=self.type,
            host=self.host,
            port=self.port,
            tls_port=self.tls_port,
            password=self.password,
            fullscreen=self.fullscreen,
            title=self.title,
            smartcard=smartcard,
            usb_auto_share=usb_auto_share,
            delete_file=delete_file,
            host_subject=self.host_subject if self.tls_port != -1 else '',
            ca=ca if self.tls_port != -1 else '',
            secure_channel=(
                'secure-channels=main;inputs;cursor;playback;record;display;usbredir;smartcard'
                if self.tls_port != -1
                else ''
            ),
            proxy=proxy_option,
            termidesk_proxy=termidesk_proxy_option,
            termidesk_clipboard=self.termidesk_clipboard_option,
            termidesk_filecopy=self.termidesk_filecopy_option,
            termidesk_pcsc=self.termidesk_pcsc_option,
        )

    @property
    def termidesk_clipboard_option(self):
        if self._termidesk_clipboard:
            return 'enable-clipboard={}'.format(self._termidesk_clipboard)
        return ''

    @property
    def termidesk_filecopy_option(self):
        if self._termidesk_filecopy:
            return 'enable-filecopy={}'.format(self._termidesk_filecopy)
        return ''

    @property
    def termidesk_pcsc_option(self):
        if self._termidesk_pcsc:
            return 'enable-pcsc={}'.format(self._termidesk_pcsc)
        return ''
