"""Модуль инициализации протокола доставки Loudplay"""
# -*- encoding: utf-8 -*-

from .direct import LoudplayDirectTransport
from .tunnel import LoudplayTunneledTransport
from uds.core.util import Config

loudplay_transports_list = [LoudplayDirectTransport, LoudplayTunneledTransport]

EXPERIMENTAL_LOUDPLAY_TRANSPORTS_ENABLED = Config.Config.section(Config.EXPERIMENTAL_SECTION). \
    value('experimental.loudplay.transports.enabled',
          '0',
          type=Config.Config.BOOLEAN_FIELD)

EXPERIMENTAL_LOUDPLAY_TRANSPORTS_ENABLED.getBool()


def transport_exclude_filter():
    """
    Возвращает фильтр, для исключения транспортов LoudPlay, если они не активированы.
    Предназначен для использования с функцией itertools.filterfalse.
    """
    loudplay_transports_enabled = EXPERIMENTAL_LOUDPLAY_TRANSPORTS_ENABLED.getBool(force=True)

    # TSRV-319: Если системная настройка `EXPERIMENTAL_LOUDPLAY_TRANSPORTS_ENABLED` выключена,
    # то необходимо исключить из выдачи транспорты Loudplay
    def exclude_filter(item):
        if not loudplay_transports_enabled:
            if item in loudplay_transports_list:
                return True
            return False
        return False
    return exclude_filter
