# -*- coding: utf-8 -*-

#
# Copyright (c) 2012 Virtual Cable S.L.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#    * Neither the name of Virtual Cable S.L. nor the names of its contributors
#      may be used to endorse or promote products derived from this software
#      without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'''
@author: Adolfo Gómez, dkmaster at dkmon dot com
'''
from __future__ import unicode_literals

import logging

from django.db import transaction

from uds.core.db.LockingManager import locking
from uds.models.UniqueId import UniqueId

from .UniqueIDGenerator import UniqueIDGenerator

logger = logging.getLogger(__name__)


class UniqueNameGenerator:
    """Генератор уникальных имен рабочих мест"""

    # Уникальный независимый ID для представления ссылки на класс в системах хранения.
    # Например, в БД.
    # Не должен меняться, но при необходимости изменения нужно будет делать миграцию данных в БД.
    # Обязательно должен переопределяться в наследниках.
    unique_id = '188d7223-38df-4eb3-a31e-9d809a02a063'  # uuid4
    label = "С переиспользованием имен"

    default_min_val = 0

    def __init__(self, owner):
        self.generator = UniqueIDGenerator('name', owner)

    def get(self, base_name, length=5):
        self.generator.setBaseName(base_name)

        with transaction.atomic(), locking(UniqueId):
            min_val = self.get_min_val(base_name)
            max_val = 10 ** length - 1
            seq = self.generator.get(min_val, max_val)

        if seq == -1:
            raise KeyError('No more names available. Please, increase service digits.')

        return "%s%0*d" % (base_name, length, seq)

    def transfer(self, base_name, name, to_uid_gen):
        self.generator.setBaseName(base_name)
        self.generator.transfer(int(name[len(base_name):]), to_uid_gen)

    def free(self, base_name, name):
        self.generator.setBaseName(base_name)
        self.generator.free(int(name[len(base_name):]))

    def release(self):
        return self.generator.release()

    def get_min_val(self, _base_name):
        return self.default_min_val
