# Generated by Django 2.2.17 on 2022-08-31 13:22

import json
import codecs
from django.db import migrations


def code(apps, _schema_editor):
    """Переносит значение DeployedService.osmanager.data['idle']
    в значение политики ServiceIdlePolicy
    """
    DeployedService = apps.get_model('uds', 'DeployedService')
    PolicyDeployedService = apps.get_model('termidesk', 'PolicyDeployedService')

    for deployed_service in DeployedService.objects.all():
        if deployed_service.osmanager is None:
            continue

        os_manager_data = codecs.decode(deployed_service.osmanager.data.encode(), 'base64').decode()
        os_manager_data = os_manager_data.split('\t')

        version = next(iter(os_manager_data), None)
        if version == 'v3':
            value = os_manager_data[2]
        else:
            continue

        data = {'value': value, 'version': 1}
        data_str = json.dumps(data)
        data_str = codecs.encode(data_str.encode(), 'base64').decode()
        PolicyDeployedService.objects.get_or_create(
            deployed_service=deployed_service,
            data_type='ServiceIdlePolicy',
            defaults={
                'name': 'Политика простоя рабочего места',
                'data': data_str,
            },
        )


def reverse_code(apps, _schema_editor):
    """Переносит значение политики ServiceIdlePolicy
    в DeployedService.osmanager.data['idle']
    """
    OsManager = apps.get_model('uds', 'OsManager')
    DeployedService = apps.get_model('uds', 'DeployedService')
    PolicyDeployedService = apps.get_model('termidesk', 'PolicyDeployedService')

    policies = PolicyDeployedService.objects.filter(data_type='ServiceIdlePolicy')
    for policy in policies:
        data_str = codecs.decode(policy.data.encode(), 'base64').decode()
        data = json.loads(data_str)
        idle = data.get('value', '-1')
        deployed_service = DeployedService.objects.filter(id=policy.deployed_service_id).first()

        if deployed_service is None or deployed_service.osmanager is None:
            continue

        os_manager_data = codecs.decode(deployed_service.osmanager.data.encode(), 'base64').decode()
        os_manager_data = os_manager_data.split('\t')
        version = next(iter(os_manager_data), None)
        if version == 'v3':
            os_manager_data[2] = idle
        else:
            continue

        os_manager_data = '\t'.join(os_manager_data)
        os_manager_data = codecs.encode(os_manager_data.encode(), 'base64').decode()
        OsManager.objects.filter(id=deployed_service.osmanager.id).update(data=os_manager_data)


class Migration(migrations.Migration):

    dependencies = [
        ('termidesk', '0027_auto_20220726_1133'),
        ('uds', '0052_auto_20220830_1603'),
    ]

    operations = [
        migrations.RunPython(code, reverse_code, elidable=True)
    ]
