# Generated by Django 2.2.17 on 2022-09-16 16:25

import codecs
import json

from django.db import migrations

# Всевозможные на данный момент типы менеджеров ОС
base_managers = [
    'LinuxManager',
    'WindowsManager',
]
domain_managers = [
    'LinuxDomainManager',
    'WinDomainManager',
    'LinuxDomainManagerALD',
    'LinuxDomainManagerIPA',
]


def manager_data_to_policy_data(os_manager):
    """Переводит параметр idle из формата для менеджеров ОС в формат для политики.

    Для базовых менеджеров "LinuxManager" и "WindowsManager" предварительные действия не нужны,
    данные располагаются в виде массива
    ['v3', self._onLogout, str(idle_normalize(int(self._idle))), str(self._max_session_time)]

    Для доменных менеджеров "LinuxDomainManager", "WinDomainManager", "LinuxDomainManagerALD" и
    "LinuxDomainManagerIPA" данные имеют вложенную структуру: значения базового менеджера
    дополнительно вкладываются в последний элемент массива доменного менеджера.
    ['v1', self._domain, self._ou, self._account, CryptoManager.manager().encrypt(self._password), base]
    или
    ['v1', self._auth_id, self._account, CryptoManager.manager().encrypt(self._password), base]
    Поэтому нужно предварительно достать нужный элемент, который содержит строку базового менеджера.
    """
    os_manager_data = codecs.decode(os_manager.data.encode(), 'base64').decode()
    os_manager_data = os_manager_data.split('\t')

    if os_manager.data_type in domain_managers:
        if len(os_manager_data) in [5, 6] and os_manager_data[0] == 'v1':
            os_manager_data = codecs.decode(os_manager_data[-1].encode(), 'hex').decode()
            os_manager_data = os_manager_data.split('\t')

    if len(os_manager_data) == 4 and os_manager_data[0] == 'v3':
        value = os_manager_data[2]
    else:
        return None

    data = {'value': value, 'version': 1}
    data_str = json.dumps(data)
    return codecs.encode(data_str.encode(), 'base64').decode()


def code(apps, _schema_editor):
    """Переносит значение DeployedService.osmanager.data['idle']
    в значение политики ServiceIdlePolicy
    """
    DeployedService = apps.get_model('uds', 'DeployedService')
    PolicyDeployedService = apps.get_model('termidesk', 'PolicyDeployedService')

    for deployed_service in DeployedService.objects.all():
        if deployed_service.osmanager is None:
            continue

        data_str = manager_data_to_policy_data(deployed_service.osmanager)

        if data_str is None:
            continue

        PolicyDeployedService.objects.get_or_create(
            deployed_service=deployed_service,
            data_type='ServiceIdlePolicy',
            defaults={
                'name': 'Политика простоя рабочего места',
                'data': data_str,
            },
        )


class Migration(migrations.Migration):

    dependencies = [
        ('uds', '0053_auto_20220831_1322'),
        ('termidesk', '0027_auto_20220726_1133'),
    ]

    operations = [
        migrations.RunPython(
            code=code,
            reverse_code=migrations.RunPython.noop,
        ),
    ]
