from __future__ import unicode_literals

from django.utils.translation import gettext_noop as _
from uds.core.services import Service
from uds.core.services import types as serviceTypes

from .RDSDeployment import RDSDeployment

import logging

logger = logging.getLogger(__name__)


class RDSService(Service):
    """This class represents the MS RDS service

    In this class we provide:
       * The Service functionality
       * The basic configuration parameters for the service
       * The form fields needed by administrators to configure this service
    """
    # : Name to show the administrator
    typeName = _('RDS Service')
    # : Type used internally to identify this service
    typeType = 'RDSService'
    # : Description shown at administration interface for this service
    typeDescription = _('RDS service')
    # : Icon file used as icon for this service
    iconFile = 'service.png'

    # : If the service provides more than 1 "deployed user" (-1 == no limit)
    maxDeployed = -1
    # : If we need to generate "cache" for this service, so users can access the
    # : provided services faster. Is usesCache is True, you will need also
    # : set publicationType.
    usesCache = False
    # : If we need to generate a "Level 2" cache for this service (i.e., L1
    # : could be running machines and L2 suspended machines)
    usesCache_L2 = False
    # : If the service needs a s.o. manager (managers are related to agents
    # : provided by services itself, i.e. virtual machines with actors)
    needsManager = False
    # : If true, the system can't do an automatic assignation of a deployed user
    # : service from this service
    mustAssignManually = False

    # : Types of deploys (services in cache and/or assigned to users)
    deployedType = RDSDeployment

    servicesTypeProvided = (serviceTypes.VDI,)

    def getHostname(self):
        """Get an IP/FQDN of a MS RDS server from Provider"""
        return self.parent().remoteHost.value

    def getMachineState(self):
        """Get a MS RDS current state"""
        return self.parent().getMachineState()

    def getUniqueId(self):
        """Returns the unique id == session id on MS RDS server"""
        return self.parent().getUniqueId()
